@@ -1313,9 +1313,9 @@ void EditDirectMessagesPriceBox(
 	}, box->lifetime());
 
 	box->addButton(tr::lng_settings_save(), [=] {
-		const auto weak = Ui::MakeWeak(box);
+		const auto weak = base::make_weak(box);
 		callback(toggle->toggled() ? *result : std::optional<int>());
-		if (const auto strong = weak.data()) {
+		if (const auto strong = weak.get()) {
 			strong->closeBox();
 		}
 	});
