@@ -18,7 +18,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/peers/replace_boost_box.h" // BoostsForGift.
 #include "boxes/premium_preview_box.h" // ShowPremiumPreviewBox.
 #include "boxes/star_gift_box.h" // ShowStarGiftBox.
-#include "boxes/transfer_gift_box.h" // ShowTransferGiftBox.
 #include "core/ui_integration.h"
 #include "data/data_boosts.h"
 #include "data/data_changes.h"
@@ -77,6 +76,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 namespace {
 
 constexpr auto kRarityTooltipDuration = 3 * crl::time(1000);
+constexpr auto kHorizontalBar = QChar(0x2015);
 
 [[nodiscard]] QString CreateMessageLink(
 		not_null<Main::Session*> session,
@@ -96,6 +96,10 @@ constexpr auto kRarityTooltipDuration = 3 * crl::time(1000);
 	return QString();
 };
 
+[[nodiscard]] QString FixupTransactionId(QString origin) {
+	return origin.replace(kHorizontalBar, QChar('-'));
+}
+
 [[nodiscard]] Data::GiftCodeLink MakeGiftCodeLink(
 		not_null<Main::Session*> session,
 		const QString &slug) {
@@ -136,9 +140,10 @@ constexpr auto kRarityTooltipDuration = 3 * crl::time(1000);
 
 [[nodiscard]] object_ptr<Ui::FlatLabel> MakeMaybeMultilineTokenValue(
 		not_null<Ui::TableLayout*> table,
-		const QString &token,
+		QString token,
 		Settings::CreditsEntryBoxStyleOverrides st) {
 	constexpr auto kOneLineCount = 24;
+	token = token.replace(QChar('-'), kHorizontalBar);
 	const auto oneLine = token.length() <= kOneLineCount;
 	return object_ptr<Ui::FlatLabel>(
 		table,
@@ -412,7 +417,7 @@ void AddTableRow(
 		table->st().defaultValue.style.font->height);
 	const auto label = Ui::CreateChild<Ui::FlatLabel>(
 		raw,
-		Lang::FormatStarsAmountDecimal(entry.credits),
+		Lang::FormatCreditsAmountDecimal(entry.credits),
 		table->st().defaultValue,
 		st::defaultPopupMenu);
 
@@ -1336,7 +1341,13 @@ void AddStarGiftTable(
 		}, tooltip->lifetime());
 	};
 
-	if (unique && entry.bareGiftOwnerId) {
+	if (unique && entry.bareGiftResaleRecipientId) {
+		AddTableRow(
+			table,
+			tr::lng_credits_box_history_entry_peer(),
+			MakePeerTableValue(table, show, PeerId(entry.bareGiftResaleRecipientId)),
+			st::giveawayGiftCodePeerMargin);
+	} else if (unique && entry.bareGiftOwnerId) {
 		const auto ownerId = PeerId(entry.bareGiftOwnerId);
 		const auto was = std::make_shared<std::optional<CollectibleId>>();
 		const auto handleChange = [=](
@@ -1376,7 +1387,7 @@ void AddStarGiftTable(
 			auto label = MakeMaybeMultilineTokenValue(table, address, st);
 			label->setClickHandlerFilter([=](const auto &...) {
 				TextUtilities::SetClipboardText(
-					TextForMimeData::Simple(address));
+					TextForMimeData::Simple(FixupTransactionId(address)));
 				show->showToast(
 					tr::lng_gift_unique_address_copied(tr::now));
 				return false;
@@ -1621,7 +1632,17 @@ void AddCreditsHistoryEntryTable(
 	const auto starrefRecipientId = PeerId(entry.starrefRecipientId);
 	const auto session = &show->session();
 	if (entry.starrefCommission) {
-		if (entry.starrefAmount) {
+		if (entry.giftResale && entry.starrefCommission < 1000) {
+			const auto full = int(base::SafeRound(entry.credits.value()
+				/ (1. - (entry.starrefCommission / 1000.))));
+			auto value = Ui::Text::IconEmoji(&st::starIconEmojiColored);
+			const auto starsText = Lang::FormatCreditsAmountDecimal(
+				CreditsAmount{ full });
+			AddTableRow(
+				table,
+				tr::lng_credits_box_history_entry_gift_full_price(),
+				rpl::single(value.append(' ' + starsText)));
+		} else if (entry.starrefAmount) {
 			AddTableRow(
 				table,
 				tr::lng_star_ref_commission_title(),
@@ -1635,7 +1656,7 @@ void AddCreditsHistoryEntryTable(
 					Ui::Text::WithEntities));
 		}
 	}
-	if (starrefRecipientId && entry.starrefAmount) {
+	if (starrefRecipientId && entry.starrefAmount && !entry.giftResale) {
 		AddTableRow(
 			table,
 			tr::lng_credits_box_history_entry_affiliate(),
@@ -1645,7 +1666,9 @@ void AddCreditsHistoryEntryTable(
 	if (peerId && entry.starrefCommission) {
 		AddTableRow(
 			table,
-			(entry.starrefAmount
+			(entry.giftResale
+				? tr::lng_credits_box_history_entry_gift_sold_to
+				: entry.starrefAmount
 				? tr::lng_credits_box_history_entry_referred
 				: tr::lng_credits_box_history_entry_miniapp)(),
 			show,
@@ -1656,6 +1679,8 @@ void AddCreditsHistoryEntryTable(
 			? tr::lng_credits_box_history_entry_referred()
 			: entry.in
 			? tr::lng_credits_box_history_entry_peer_in()
+			: entry.giftResale
+			? tr::lng_credits_box_history_entry_gift_bought_from()
 			: entry.giftUpgraded
 			? tr::lng_credits_box_history_entry_gift_from()
 			: tr::lng_credits_box_history_entry_peer();
@@ -1710,7 +1735,7 @@ void AddCreditsHistoryEntryTable(
 			(entry.gift
 				? tr::lng_credits_box_history_entry_peer_in
 				: tr::lng_credits_box_history_entry_via)(),
-			(entry.gift
+			((entry.gift && entry.credits.stars())
 				? tr::lng_credits_box_history_entry_anonymous
 				: tr::lng_credits_box_history_entry_fragment)(
 					Ui::Text::RichLangValue));
@@ -1768,7 +1793,7 @@ void AddCreditsHistoryEntryTable(
 		auto value = Ui::Text::IconEmoji(&st::starIconEmojiColored);
 		const auto full = (entry.in ? 1 : -1)
 			* (entry.credits + entry.paidMessagesAmount);
-		const auto starsText = Lang::FormatStarsAmountDecimal(full);
+		const auto starsText = Lang::FormatCreditsAmountDecimal(full);
 		AddTableRow(
 			table,
 			tr::lng_credits_paid_messages_full(),
@@ -1787,7 +1812,7 @@ void AddCreditsHistoryEntryTable(
 		auto label = MakeMaybeMultilineTokenValue(table, entry.id, st);
 		label->setClickHandlerFilter([=](const auto &...) {
 			TextUtilities::SetClipboardText(
-				TextForMimeData::Simple(entry.id));
+				TextForMimeData::Simple(FixupTransactionId(entry.id)));
 			show->showToast(
 				tr::lng_credits_box_history_entry_id_copied(tr::now));
 			return false;
@@ -1962,3 +1987,29 @@ void AddCreditsBoostTable(
 			rpl::single(Ui::Text::WithEntities(langDateTime(b.expiresAt))));
 	}
 }
+
+void AddChannelEarnTable(
+		std::shared_ptr<Ui::Show> show,
+		not_null<Ui::VerticalLayout*> container,
+		const Data::CreditsHistoryEntry &entry) {
+	const auto table = container->add(
+		object_ptr<Ui::TableLayout>(
+			container,
+			st::giveawayGiftCodeTable),
+		st::giveawayGiftCodeTableMargin);
+	if (!entry.id.isEmpty()) {
+		auto label = MakeMaybeMultilineTokenValue(table, entry.id, {});
+		label->setClickHandlerFilter([=](const auto &...) {
+			TextUtilities::SetClipboardText(
+				TextForMimeData::Simple(FixupTransactionId(entry.id)));
+			show->showToast(
+				tr::lng_credits_box_history_entry_id_copied(tr::now));
+			return false;
+		});
+		AddTableRow(
+			table,
+			tr::lng_credits_box_history_entry_id(),
+			std::move(label),
+			st::giveawayGiftCodeValueMargin);
+	}
+}
