@@ -11,6 +11,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/admin_log/history_admin_log_filter.h"
 #include "core/ui_integration.h"
 #include "data/stickers/data_custom_emoji.h"
+#include "data/business/data_business_chatbots.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
 #include "data/data_session.h"
@@ -52,6 +53,7 @@ namespace {
 constexpr auto kSlowmodeValues = 7;
 constexpr auto kBoostsUnrestrictValues = 5;
 constexpr auto kForceDisableTooltipDuration = 3 * crl::time(1000);
+constexpr auto kDefaultChargeStars = 10;
 
 [[nodiscard]] auto Dependencies(PowerSaving::Flags)
 -> std::vector<std::pair<PowerSaving::Flag, PowerSaving::Flag>> {
@@ -63,6 +65,11 @@ constexpr auto kForceDisableTooltipDuration = 3 * crl::time(1000);
 	return std::vector<std::pair<Flag, Flag>>{};
 }
 
+[[nodiscard]] auto Dependencies(Data::ChatbotsPermissions) {
+	using Flag = Data::ChatbotsPermission;
+	return std::vector<std::pair<Flag, Flag>>{};
+}
+
 [[nodiscard]] auto NestedRestrictionLabelsList(
 	Data::RestrictionsSetOptions options)
 -> std::vector<NestedEditFlagsLabels<ChatRestrictions>> {
@@ -157,11 +164,15 @@ constexpr auto kForceDisableTooltipDuration = 3 * crl::time(1000);
 	auto stories = std::vector<AdminRightLabel>{
 		{ Flag::PostStories, tr::lng_rights_channel_post_stories(tr::now) },
 		{ Flag::EditStories, tr::lng_rights_channel_edit_stories(tr::now) },
-		{ Flag::DeleteStories, tr::lng_rights_channel_delete_stories(tr::now) },
+		{
+			Flag::DeleteStories,
+			tr::lng_rights_channel_delete_stories(tr::now),
+		},
 	};
 	auto second = std::vector<AdminRightLabel>{
 		{ Flag::InviteByLinkOrAdd, tr::lng_rights_group_invite(tr::now) },
 		{ Flag::ManageCall, tr::lng_rights_channel_manage_calls(tr::now) },
+		{ Flag::ManageDirect, tr::lng_rights_channel_manage_direct(tr::now) },
 		{ Flag::AddAdmins, tr::lng_rights_add_admins(tr::now) },
 	};
 	return {
@@ -678,7 +689,7 @@ template <typename Flags>
 				checkView->setChecked(false, anim::type::instant);
 			} else if (locked.has_value()) {
 				if (checked != toggled) {
-					if (!state->toast) {
+					if (!state->toast && !locked->isEmpty()) {
 						state->toast = Ui::Toast::Show(container, {
 							.text = { *locked },
 							.duration = kForceDisableTooltipDuration,
@@ -1132,7 +1143,7 @@ void ShowEditPeerPermissionsBox(
 				result.emplace(
 					Flag::ChangeInfo | Flag::PinMessages,
 					tr::lng_rights_permission_unavailable(tr::now));
-			} else if (channel->isMegagroup() && channel->linkedChat()) {
+			} else if (channel->isMegagroup() && channel->discussionLink()) {
 				result.emplace(
 					Flag::ChangeInfo | Flag::PinMessages,
 					tr::lng_rights_permission_in_discuss(tr::now));
@@ -1168,7 +1179,7 @@ void ShowEditPeerPermissionsBox(
 	if (available) {
 		Ui::AddSkip(inner);
 		const auto starsPerMessage = peer->isChannel()
-			? peer->asChannel()->starsPerMessage()
+			? peer->asChannel()->commonStarsPerMessage()
 			: 0;
 		charging = inner->add(object_ptr<Ui::SettingsButton>(
 			inner,
@@ -1190,7 +1201,8 @@ void ShowEditPeerPermissionsBox(
 		state->starsPerMessage = SetupChargeSlider(
 			chargeInner,
 			peer,
-			starsPerMessage);
+			(starsPerMessage > 0) ? starsPerMessage : std::optional<int>(),
+			kDefaultChargeStars);
 	}
 
 	static constexpr auto kSendRestrictions = Flag::EmbedLinks
@@ -1488,3 +1500,20 @@ EditFlagsControl<AdminLog::FilterValue::Flags> CreateEditAdminLogFilter(
 
 	return result;
 }
+
+EditFlagsControl<Data::ChatbotsPermissions> CreateEditChatbotPermissions(
+		QWidget *parent,
+		Data::ChatbotsPermissions flags) {
+	auto widget = object_ptr<Ui::VerticalLayout>(parent);
+	auto descriptor = Data::ChatbotsPermissionsLabels();
+	descriptor.disabledMessages.emplace(
+		Data::ChatbotsPermission::ViewMessages,
+		QString());
+	auto result = CreateEditFlags(
+		widget.data(),
+		flags | Data::ChatbotsPermission::ViewMessages,
+		std::move(descriptor));
+	result.widget = std::move(widget);
+
+	return result;
+}
