@@ -677,7 +677,7 @@ void ShareBox::submit(Api::SendOptions options) {
 	_submitLifetime.destroy();
 
 	auto threads = _inner->selected();
-	const auto weak = Ui::MakeWeak(this);
+	const auto weak = base::make_weak(this);
 	const auto field = _comment->entity();
 	auto comment = field->getTextWithAppliedMarkdown();
 	const auto checkPaid = [=] {
@@ -1371,8 +1371,8 @@ void ShareBox::Inner::changeCheckState(Chat *chat) {
 }
 
 void ShareBox::Inner::chooseForumTopic(not_null<Data::Forum*> forum) {
-	const auto guard = Ui::MakeWeak(this);
-	const auto weak = std::make_shared<QPointer<Ui::BoxContent>>();
+	const auto guard = base::make_weak(this);
+	const auto weak = std::make_shared<base::weak_qptr<Ui::BoxContent>>();
 	auto chosen = [=](not_null<Data::ForumTopic*> topic) {
 		if (const auto strong = *weak) {
 			strong->closeBox();
@@ -1419,8 +1419,8 @@ void ShareBox::Inner::chooseForumTopic(not_null<Data::Forum*> forum) {
 
 void ShareBox::Inner::chooseMonoforumSublist(
 		not_null<Data::SavedMessages*> monoforum) {
-	const auto guard = Ui::MakeWeak(this);
-	const auto weak = std::make_shared<QPointer<Ui::BoxContent>>();
+	const auto guard = base::make_weak(this);
+	const auto weak = std::make_shared<base::weak_qptr<Ui::BoxContent>>();
 	auto chosen = [=](not_null<Data::SavedSublist*> sublist) {
 		if (const auto strong = *weak) {
 			strong->closeBox();
@@ -1953,7 +1953,7 @@ void FastShareLink(
 		std::shared_ptr<Main::SessionShow> show,
 		const QString &url,
 		ShareBoxStyleOverrides st) {
-	const auto box = std::make_shared<QPointer<Ui::BoxContent>>();
+	const auto box = std::make_shared<base::weak_qptr<Ui::BoxContent>>();
 	const auto sending = std::make_shared<bool>();
 	auto copyCallback = [=] {
 		QGuiApplication::clipboard()->setText(url);
