@@ -23,6 +23,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "main/main_session.h"
 #include "payments/payments_checkout_process.h"
 #include "ui/boxes/confirm_box.h"
+#include "ui/controls/sub_tabs.h"
+#include "ui/controls/ton_common.h"
 #include "ui/layers/generic_box.h"
 #include "ui/text/text_utilities.h"
 #include "ui/toast/toast.h"
@@ -90,8 +92,8 @@ void ConfirmExportBox(
 			Ui::Text::WithEntities),
 		st::boxLabel));
 	box->addButton(tr::lng_gift_transfer_confirm_button(), [=] {
-		confirmed([weak = Ui::MakeWeak(box)] {
-			if (const auto strong = weak.data()) {
+		confirmed([weak = base::make_weak(box)] {
+			if (const auto strong = weak.get()) {
 				strong->closeBox();
 			}
 		});
@@ -143,7 +145,7 @@ void ExportOnBlockchain(
 			fields.customSubmitButton = tr::lng_passcode_submit();
 			fields.customCheckCallback = crl::guard(parent, [=](
 					const Core::CloudPasswordResult &result,
-					QPointer<PasscodeBox> box) {
+					base::weak_qptr<PasscodeBox> box) {
 				using ExportUrl = MTPpayments_StarGiftWithdrawalUrl;
 				session->api().request(
 					MTPpayments_GetStarGiftWithdrawalUrl(
@@ -180,7 +182,7 @@ void ExportOnBlockchain(
 	const auto state = std::make_shared<State>();
 	const auto activate = [=] {
 		const auto now = base::unixtime::now();
-		const auto weak = Ui::MakeWeak(box);
+		const auto weak = base::make_weak(box);
 		const auto left = (when > now) ? (when - now) : 0;
 		const auto hours = left ? std::max((left + 1800) / 3600, 1) : 0;
 		if (!hours) {
@@ -193,7 +195,7 @@ void ExportOnBlockchain(
 					state->exporting = false;
 					close();
 				}, [=] {
-					if (const auto strong = weak.data()) {
+					if (const auto strong = weak.get()) {
 						strong->closeBox();
 					}
 					close();
@@ -425,8 +427,6 @@ void TransferGift(
 		Data::SavedStarGiftId savedId,
 		Fn<void(Payments::CheckoutResult)> done,
 		bool skipPaymentForm = false) {
-	Expects(to->isUser());
-
 	const auto session = &window->session();
 	const auto weak = base::make_weak(window);
 	auto formDone = [=](
@@ -488,6 +488,7 @@ void BuyResaleGift(
 		std::shared_ptr<ChatHelpers::Show> show,
 		not_null<PeerData*> to,
 		std::shared_ptr<Data::UniqueGift> gift,
+		CreditsType type,
 		Fn<void(Payments::CheckoutResult)> done) {
 	auto paymentDone = [=](
 			Payments::CheckoutResult result,
@@ -503,22 +504,44 @@ void BuyResaleGift(
 		}
 	};
 
+	using Flag = MTPDinputInvoiceStarGiftResale::Flag;
 	const auto invoice = MTP_inputInvoiceStarGiftResale(
+		MTP_flags((type == CreditsType::Ton) ? Flag::f_ton : Flag()),
 		MTP_string(gift->slug),
 		to->input);
 
-	Ui::RequestStarsForm(show, invoice, [=](
+	Ui::RequestOurForm(show, invoice, [=](
 			uint64 formId,
-			uint64 price,
+			CreditsAmount price,
 			std::optional<Payments::CheckoutResult> failure) {
+		if ((type == CreditsType::Ton && price.stars())
+			|| (type == CreditsType::Stars && price.ton())) {
+			paymentDone(Payments::CheckoutResult::Failed, nullptr);
+			return;
+		}
 		const auto submit = [=] {
-			SubmitStarsForm(show, invoice, formId, price, paymentDone);
+			if (price.stars()) {
+				SubmitStarsForm(
+					show,
+					invoice,
+					formId,
+					price.whole(),
+					paymentDone);
+			} else {
+				SubmitTonForm(show, invoice, formId, price, paymentDone);
+			}
 		};
+		const auto was = (type == CreditsType::Ton)
+			? Data::UniqueGiftResaleTon(*gift)
+			: Data::UniqueGiftResaleStars(*gift);
 		if (failure) {
 			paymentDone(*failure, nullptr);
-		} else if (price != gift->starsForResale) {
-			const auto cost = Ui::Text::IconEmoji(&st::starIconEmoji).append(
-				Lang::FormatCountDecimal(price));
+		} else if (price != was) {
+			const auto cost = price.ton()
+				? Ui::Text::IconEmoji(&st::tonIconEmoji).append(
+					Lang::FormatCreditsAmountDecimal(price))
+				: Ui::Text::IconEmoji(&st::starIconEmoji).append(
+					Lang::FormatCountDecimal(price.whole()));
 			const auto cancelled = [=](Fn<void()> close) {
 				paymentDone(Payments::CheckoutResult::Cancelled, nullptr);
 				close();
@@ -574,11 +597,11 @@ void ShowTransferToBox(
 				return;
 			}
 			state->sent = true;
-			const auto weak = Ui::MakeWeak(box);
+			const auto weak = base::make_weak(box);
 			const auto done = [=](Payments::CheckoutResult result) {
 				if (result == Payments::CheckoutResult::Cancelled) {
 					closeParentBox();
-					if (const auto strong = weak.data()) {
+					if (const auto strong = weak.get()) {
 						strong->closeBox();
 					}
 				} else if (result != Payments::CheckoutResult::Paid) {
@@ -588,7 +611,7 @@ void ShowTransferToBox(
 						controller->showPeerHistory(peer);
 					}
 					closeParentBox();
-					if (const auto strong = weak.data()) {
+					if (const auto strong = weak.get()) {
 						strong->closeBox();
 					}
 				}
@@ -653,30 +676,71 @@ void ShowTransferGiftBox(
 void ShowBuyResaleGiftBox(
 		std::shared_ptr<ChatHelpers::Show> show,
 		std::shared_ptr<Data::UniqueGift> gift,
+		bool forceTon,
 		not_null<PeerData*> to,
 		Fn<void()> closeParentBox) {
 	show->show(Box([=](not_null<Ui::GenericBox*> box) {
-		box->setTitle(tr::lng_gift_buy_resale_title(
-			lt_name,
-			rpl::single(UniqueGiftName(*gift))));
-
-		auto transfer = tr::lng_gift_buy_resale_button(
-			lt_cost,
-			rpl::single(
-				Ui::Text::IconEmoji(&st::starIconEmoji).append(
-					Lang::FormatCountDecimal(gift->starsForResale))),
-			Ui::Text::WithEntities);
-
 		struct State {
+			rpl::variable<bool> ton;
 			bool sent = false;
 		};
 		const auto state = std::make_shared<State>();
+		state->ton = gift->onlyAcceptTon || forceTon;
+
+		if (gift->onlyAcceptTon) {
+			box->addRow(
+				object_ptr<Ui::FlatLabel>(
+					box,
+					tr::lng_gift_buy_resale_only_ton(
+						Ui::Text::RichLangValue),
+					st::resaleConfirmTonOnly),
+				st::boxRowPadding + st::resaleConfirmTonOnlyMargin);
+		} else {
+			const auto tabs = box->addRow(
+				object_ptr<Ui::SubTabs>(
+					box,
+					Ui::SubTabsOptions{
+						.selected = (state->ton.current()
+							? u"ton"_q
+							: u"stars"_q),
+						.centered = true,
+					},
+					std::vector<Ui::SubTabsTab>{
+						{
+							u"stars"_q,
+							tr::lng_gift_buy_resale_pay_stars(
+								tr::now,
+								Ui::Text::WithEntities),
+						},
+						{
+							u"ton"_q,
+							tr::lng_gift_buy_resale_pay_ton(
+								tr::now,
+								Ui::Text::WithEntities),
+						},
+					}),
+				st::boxRowPadding + st::resaleConfirmTonOnlyMargin);
+			tabs->activated() | rpl::start_with_next([=](QString id) {
+				tabs->setActiveTab(id);
+				state->ton = (id == u"ton"_q);
+			}, tabs->lifetime());
+		}
+
+		auto transfer = state->ton.value() | rpl::map([=](bool ton) {
+			return tr::lng_gift_buy_resale_button(
+				lt_cost,
+				rpl::single(ton
+					? Data::FormatGiftResaleTon(*gift)
+					: Data::FormatGiftResaleStars(*gift)),
+				Ui::Text::WithEntities);
+		}) | rpl::flatten_latest();
+
 		auto callback = [=](Fn<void()> close) {
 			if (state->sent) {
 				return;
 			}
 			state->sent = true;
-			const auto weak = Ui::MakeWeak(box);
+			const auto weak = base::make_weak(box);
 			const auto done = [=](Payments::CheckoutResult result) {
 				if (result == Payments::CheckoutResult::Cancelled) {
 					closeParentBox();
@@ -684,33 +748,41 @@ void ShowBuyResaleGiftBox(
 				} else if (result != Payments::CheckoutResult::Paid) {
 					state->sent = false;
 				} else {
-					show->showToast(u"done!"_q);
 					closeParentBox();
 					close();
 				}
 			};
-			BuyResaleGift(show, to, gift, done);
+			const auto type = state->ton.current()
+				? CreditsType::Ton
+				: CreditsType::Stars;
+			BuyResaleGift(show, to, gift, type, done);
 		};
 
+		auto price = state->ton.value() | rpl::map([=](bool ton) {
+			return ton
+				? tr::lng_action_gift_for_ton(
+					lt_count_decimal,
+					rpl::single(gift->nanoTonForResale
+						/ float64(Ui::kNanosInOne)),
+					Ui::Text::Bold)
+				: tr::lng_action_gift_for_stars(
+					lt_count_decimal,
+					rpl::single(gift->starsForResale * 1.),
+					Ui::Text::Bold);
+		}) | rpl::flatten_latest();
 		Ui::ConfirmBox(box, {
 			.text = to->isSelf()
 				? tr::lng_gift_buy_resale_confirm_self(
 					lt_name,
 					rpl::single(Ui::Text::Bold(UniqueGiftName(*gift))),
 					lt_price,
-					tr::lng_action_gift_for_stars(
-						lt_count,
-						rpl::single(gift->starsForResale * 1.),
-						Ui::Text::Bold),
+					std::move(price),
 					Ui::Text::WithEntities)
 				: tr::lng_gift_buy_resale_confirm(
 					lt_name,
 					rpl::single(Ui::Text::Bold(UniqueGiftName(*gift))),
 					lt_price,
-					tr::lng_action_gift_for_stars(
-						lt_count,
-						rpl::single(gift->starsForResale * 1.),
-						Ui::Text::Bold),
+					std::move(price),
 					lt_user,
 					rpl::single(Ui::Text::Bold(to->shortName())),
 					Ui::Text::WithEntities),
