@@ -240,7 +240,7 @@ QByteArray Settings::serialize() const {
 		+ Serialize::stringSize(_customFontFamily)
 		+ sizeof(qint32) * 3
 		+ Serialize::bytearraySize(_tonsiteStorageToken)
-		+ sizeof(qint32) * 7;
+		+ sizeof(qint32) * 8;
 
 	auto result = QByteArray();
 	result.reserve(size);
@@ -401,7 +401,8 @@ QByteArray Settings::serialize() const {
 			<< qint32(_recordVideoMessages ? 1 : 0)
 			<< SerializeVideoQuality(_videoQuality)
 			<< qint32(_ivZoom.current())
-			<< qint32(_systemDarkModeEnabled.current() ? 1 : 0);
+			<< qint32(_systemDarkModeEnabled.current() ? 1 : 0)
+			<< qint32(_quickDialogAction);
 	}
 
 	Ensures(result.size() == size);
@@ -530,6 +531,7 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	qint32 recordVideoMessages = _recordVideoMessages ? 1 : 0;
 	quint32 videoQuality = SerializeVideoQuality(_videoQuality);
 	quint32 chatFiltersHorizontal = _chatFiltersHorizontal.current() ? 1 : 0;
+	quint32 quickDialogAction = quint32(_quickDialogAction);
 
 	stream >> themesAccentColors;
 	if (!stream.atEnd()) {
@@ -858,6 +860,9 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	if (!stream.atEnd()) {
 		stream >> systemDarkModeEnabled;
 	}
+	if (!stream.atEnd()) {
+		stream >> quickDialogAction;
+	}
 	if (stream.status() != QDataStream::Ok) {
 		LOG(("App Error: "
 			"Bad data for Core::Settings::constructFromSerialized()"));
@@ -1079,6 +1084,7 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	_recordVideoMessages = (recordVideoMessages == 1);
 	_videoQuality = DeserializeVideoQuality(videoQuality);
 	_chatFiltersHorizontal = (chatFiltersHorizontal == 1);
+	_quickDialogAction = Dialogs::Ui::QuickDialogAction(quickDialogAction);
 }
 
 QString Settings::getSoundPath(const QString &key) const {
@@ -1470,6 +1476,7 @@ void Settings::resetOnLastLogout() {
 	_recordVideoMessages = false;
 	_videoQuality = {};
 	_chatFiltersHorizontal = false;
+	_quickDialogAction = Dialogs::Ui::QuickDialogAction::Disabled;
 
 	_recentEmojiPreload.clear();
 	_recentEmoji.clear();
@@ -1657,4 +1664,12 @@ void Settings::setChatFiltersHorizontal(bool value) {
 	_chatFiltersHorizontal = value;
 }
 
+Dialogs::Ui::QuickDialogAction Settings::quickDialogAction() const {
+	return _quickDialogAction;
+}
+
+void Settings::setQuickDialogAction(Dialogs::Ui::QuickDialogAction action) {
+	_quickDialogAction = action;
+}
+
 } // namespace Core
