@@ -110,6 +110,7 @@ const auto CommandByName = base::flat_map<QString, Command>{
 	{ u"read_chat"_q         , Command::ReadChat },
 
 	{ u"show_chat_menu"_q    , Command::ShowChatMenu },
+	{ u"show_chat_preview"_q , Command::ShowChatPreview },
 
 	// Shortcuts that have no default values.
 	{ u"message"_q                       , Command::JustSendMessage },
@@ -506,6 +507,7 @@ void Manager::fillDefaults() {
 	set(u"ctrl+r"_q, Command::ReadChat);
 
 	set(u"ctrl+\\"_q, Command::ShowChatMenu);
+	set(u"ctrl+]"_q, Command::ShowChatPreview);
 
 	_defaults = keysCurrents();
 }
