@@ -28,8 +28,11 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 namespace Data {
 namespace {
 
+constexpr auto kMs = crl::time(1000);
 constexpr auto kRequestTimeLimit = 5 * 60 * crl::time(1000);
 
+const auto kFlaggedPreload = ((MediaPreload*)quintptr(0x01));
+
 [[nodiscard]] bool TooEarlyForRequest(crl::time received) {
 	return (received > 0) && (received + kRequestTimeLimit > crl::now());
 }
@@ -73,17 +76,21 @@ void SponsoredMessages::clear() {
 
 void SponsoredMessages::clearOldRequests() {
 	const auto now = crl::now();
-	while (true) {
-		const auto i = ranges::find_if(_requests, [&](const auto &value) {
-			const auto &request = value.second;
-			return !request.requestId
-				&& (request.lastReceived + kRequestTimeLimit <= now);
-		});
-		if (i == end(_requests)) {
-			break;
+	const auto clear = [&](auto &requests) {
+		while (true) {
+			const auto i = ranges::find_if(requests, [&](const auto &value) {
+				const auto &request = value.second;
+				return !request.requestId
+					&& (request.lastReceived + kRequestTimeLimit <= now);
+			});
+			if (i == end(requests)) {
+				break;
+			}
+			requests.erase(i);
 		}
-		_requests.erase(i);
-	}
+	};
+	clear(_requests);
+	clear(_requestsForVideo);
 }
 
 SponsoredMessages::AppendResult SponsoredMessages::append(
@@ -232,6 +239,11 @@ bool SponsoredMessages::canHaveFor(not_null<History*> history) const {
 	return false;
 }
 
+bool SponsoredMessages::canHaveFor(not_null<HistoryItem*> item) const {
+	return item->history()->peer->isBroadcast()
+		&& item->isRegular();
+}
+
 bool SponsoredMessages::isTopBarFor(not_null<History*> history) const {
 	if (peerIsUser(history->peer->id)) {
 		if (const auto user = history->peer->asUser()) {
@@ -277,6 +289,78 @@ void SponsoredMessages::request(not_null<History*> history, Fn<void()> done) {
 	}).send();
 }
 
+void SponsoredMessages::requestForVideo(
+		not_null<HistoryItem*> item,
+		Fn<void(SponsoredForVideo)> done) {
+	Expects(done != nullptr);
+
+	if (!canHaveFor(item)) {
+		done({});
+		return;
+	}
+	const auto peer = item->history()->peer;
+	auto &request = _requestsForVideo[peer];
+	if (TooEarlyForRequest(request.lastReceived)) {
+		auto prepared = prepareForVideo(peer);
+		if (prepared.list.empty()
+			|| prepared.state.itemIndex < prepared.list.size()
+			|| prepared.state.leftTillShow > 0) {
+			done(std::move(prepared));
+			return;
+		}
+	}
+	request.callbacks.push_back(std::move(done));
+	if (request.requestId) {
+		return;
+	}
+	{
+		const auto it = _dataForVideo.find(peer);
+		if (it != end(_dataForVideo)) {
+			auto &list = it->second;
+			// Don't rebuild currently displayed messages.
+			const auto proj = [](const Entry &e) {
+				return e.item != nullptr;
+			};
+			if (ranges::any_of(list.entries, proj)) {
+				return;
+			}
+		}
+	}
+	const auto finish = [=] {
+		const auto i = _requestsForVideo.find(peer);
+		if (i != end(_requestsForVideo)) {
+			for (const auto &callback : base::take(i->second.callbacks)) {
+				callback(prepareForVideo(peer));
+			}
+		}
+	};
+	using Flag = MTPmessages_GetSponsoredMessages::Flag;
+	request.requestId = _session->api().request(
+		MTPmessages_GetSponsoredMessages(
+			MTP_flags(Flag::f_msg_id),
+			peer->input,
+			MTP_int(item->id.bare))
+	).done([=](const MTPmessages_sponsoredMessages &result) {
+		parseForVideo(peer, result);
+		finish();
+	}).fail([=] {
+		_requestsForVideo.remove(peer);
+		finish();
+	}).send();
+}
+
+void SponsoredMessages::updateForVideo(
+		FullMsgId itemId,
+		SponsoredForVideoState state) {
+	if (state.initial()) {
+		return;
+	}
+	const auto i = _dataForVideo.find(_session->data().peer(itemId.peer));
+	if (i != end(_dataForVideo)) {
+		i->second.state = state;
+	}
+}
+
 void SponsoredMessages::parse(
 		not_null<History*> history,
 		const MTPmessages_sponsoredMessages &list) {
@@ -292,12 +376,9 @@ void SponsoredMessages::parse(
 		_session->data().processChats(data.vchats());
 
 		const auto &messages = data.vmessages().v;
-		auto &list = _data.emplace(history, List()).first->second;
+		auto &list = _data.emplace(history).first->second;
 		list.entries.clear();
 		list.received = crl::now();
-		for (const auto &message : messages) {
-			append(history, list, message);
-		}
 		if (const auto postsBetween = data.vposts_between()) {
 			list.postsBetween = postsBetween->v;
 			list.state = State::InjectToMiddle;
@@ -306,10 +387,61 @@ void SponsoredMessages::parse(
 				? State::AppendToEnd
 				: State::AppendToTopBar;
 		}
+		for (const auto &message : messages) {
+			append([=] {
+				return &_data[history].entries;
+			}, history, message);
+		}
 	}, [](const MTPDmessages_sponsoredMessagesEmpty &) {
 	});
 }
 
+void SponsoredMessages::parseForVideo(
+		not_null<PeerData*> peer,
+		const MTPmessages_sponsoredMessages &list) {
+	auto &request = _requestsForVideo[peer];
+	request.lastReceived = crl::now();
+	request.requestId = 0;
+	if (!_clearTimer.isActive()) {
+		_clearTimer.callOnce(kRequestTimeLimit * 2);
+	}
+
+	list.match([&](const MTPDmessages_sponsoredMessages &data) {
+		_session->data().processUsers(data.vusers());
+		_session->data().processChats(data.vchats());
+
+		const auto history = _session->data().history(peer);
+		const auto &messages = data.vmessages().v;
+		auto &list = _dataForVideo.emplace(peer).first->second;
+		list.entries.clear();
+		list.received = crl::now();
+		list.startDelay = data.vstart_delay().value_or_empty() * kMs;
+		list.betweenDelay = data.vbetween_delay().value_or_empty() * kMs;
+		for (const auto &message : messages) {
+			append([=] {
+				return &_dataForVideo[peer].entries;
+			}, history, message);
+		}
+	}, [](const MTPDmessages_sponsoredMessagesEmpty &) {
+	});
+}
+
+SponsoredForVideo SponsoredMessages::prepareForVideo(
+		not_null<PeerData*> peer) {
+	const auto i = _dataForVideo.find(peer);
+	if (i == end(_dataForVideo) || i->second.entries.empty()) {
+		return {};
+	}
+	return SponsoredForVideo{
+		.list = i->second.entries | ranges::views::transform(
+			&Entry::sponsored
+		) | ranges::to_vector,
+		.startDelay = i->second.startDelay,
+		.betweenDelay = i->second.betweenDelay,
+		.state = i->second.state,
+	};
+}
+
 FullMsgId SponsoredMessages::fillTopBar(
 		not_null<History*> history,
 		not_null<Ui::RpWidget*> widget) {
@@ -359,8 +491,8 @@ rpl::producer<> SponsoredMessages::itemRemoved(const FullMsgId &fullId) {
 }
 
 void SponsoredMessages::append(
+		Fn<not_null<std::vector<Entry>*>()> entries,
 		not_null<History*> history,
-		List &list,
 		const MTPSponsoredMessage &message) {
 	const auto &data = message.data();
 	const auto randomId = data.vrandom_id().v;
@@ -371,14 +503,14 @@ void SponsoredMessages::append(
 			data.vmedia()->match([&](const MTPDmessageMediaPhoto &media) {
 				if (const auto tlPhoto = media.vphoto()) {
 					tlPhoto->match([&](const MTPDphoto &data) {
-						mediaPhoto = history->owner().processPhoto(data);
+						mediaPhoto = _session->data().processPhoto(data);
 					}, [](const MTPDphotoEmpty &) {
 					});
 				}
 			}, [&](const MTPDmessageMediaDocument &media) {
 				if (const auto tlDocument = media.vdocument()) {
 					tlDocument->match([&](const MTPDdocument &data) {
-						const auto d = history->owner().processDocument(
+						const auto d = _session->data().processDocument(
 							data,
 							media.valt_documents());
 						if (d->isVideoFile()
@@ -399,7 +531,7 @@ void SponsoredMessages::append(
 		.link = qs(data.vurl()),
 		.buttonText = qs(data.vbutton_text()),
 		.photoId = data.vphoto()
-			? history->session().data().processPhoto(*data.vphoto())->id
+			? _session->data().processPhoto(*data.vphoto())->id
 			: PhotoId(0),
 		.mediaPhotoId = (mediaPhoto ? mediaPhoto->id : 0),
 		.mediaDocumentId = (mediaDocument ? mediaDocument->id : 0),
@@ -435,25 +567,24 @@ void SponsoredMessages::append(
 		.link = from.link,
 		.sponsorInfo = std::move(sponsorInfo),
 		.additionalInfo = std::move(additionalInfo),
+		.durationMin = data.vmin_display_duration().value_or_empty() * kMs,
+		.durationMax = data.vmax_display_duration().value_or_empty() * kMs,
 	};
-	list.entries.push_back({
-		.sponsored = std::move(sharedMessage),
-	});
-	auto &entry = list.entries.back();
-	const auto itemId = entry.itemFullId = FullMsgId(
+	const auto itemId = FullMsgId(
 		history->peer->id,
 		_session->data().nextLocalMessageId());
+	const auto list = entries();
+	list->push_back({
+		.itemFullId = itemId,
+		.sponsored = std::move(sharedMessage),
+	});
+	auto &entry = list->back();
 	const auto fileOrigin = FileOrigin(); // No way to refresh in ads.
 
-	static const auto kFlaggedPreload = ((MediaPreload*)quintptr(0x01));
 	const auto preloaded = [=] {
-		const auto i = _data.find(history);
-		if (i == end(_data)) {
-			return;
-		}
-		auto &entries = i->second.entries;
-		const auto j = ranges::find(entries, itemId, &Entry::itemFullId);
-		if (j == end(entries)) {
+		const auto list = entries();
+		const auto j = ranges::find(*list, itemId, &Entry::itemFullId);
+		if (j == end(*list)) {
 			return;
 		}
 		auto &entry = *j;
@@ -551,7 +682,11 @@ SponsoredMessages::Details SponsoredMessages::lookupDetails(
 	if (!entryPtr) {
 		return {};
 	}
-	const auto &data = entryPtr->sponsored;
+	return lookupDetails(entryPtr->sponsored);
+}
+
+SponsoredMessages::Details SponsoredMessages::lookupDetails(
+		const SponsoredMessage &data) const {
 	return {
 		.info = Prepare(data),
 		.link = data.link,
