@@ -685,7 +685,7 @@ bool PeerData::canCreatePolls() const {
 }
 
 bool PeerData::canCreateTodoLists() const {
-	if (isMonoforum()) {
+	if (isMonoforum() || isBroadcast()) {
 		return false;
 	}
 	return session().premium()
@@ -709,6 +709,27 @@ bool PeerData::canManageTopics() const {
 	return false;
 }
 
+bool PeerData::canPostStories() const {
+	if (const auto channel = asChannel()) {
+		return channel->canPostStories();
+	}
+	return isSelf();
+}
+
+bool PeerData::canEditStories() const {
+	if (const auto channel = asChannel()) {
+		return channel->canEditStories();
+	}
+	return isSelf();
+}
+
+bool PeerData::canDeleteStories() const {
+	if (const auto channel = asChannel()) {
+		return channel->canDeleteStories();
+	}
+	return isSelf();
+}
+
 bool PeerData::canManageGifts() const {
 	if (const auto channel = asChannel()) {
 		return channel->canPostMessages();
@@ -1690,6 +1711,13 @@ int PeerData::starsPerMessageChecked() const {
 	return starsPerMessage();
 }
 
+Data::StarsRating PeerData::starsRating() const {
+	if (const auto user = asUser()) {
+		return user->starsRating();
+	}
+	return {};
+}
+
 Data::GroupCall *PeerData::groupCall() const {
 	if (const auto chat = asChat()) {
 		return chat->groupCall();
