@@ -44,9 +44,12 @@ struct UniqueGift {
 	QString ownerAddress;
 	QString ownerName;
 	PeerId ownerId = 0;
-	int number = 0;
-	int starsForTransfer = -1;
+	PeerData *releasedBy = nullptr;
+	int64 nanoTonForResale = -1;
 	int starsForResale = -1;
+	int starsForTransfer = -1;
+	int number = 0;
+	bool onlyAcceptTon = false;
 	TimeId exportAt = 0;
 	TimeId canTransferAt = 0;
 	TimeId canResellAt = 0;
@@ -56,9 +59,15 @@ struct UniqueGift {
 	UniqueGiftOriginalDetails originalDetails;
 };
 
-[[nodiscard]] inline QString UniqueGiftName(const UniqueGift &gift) {
-	return gift.title + u" #"_q + QString::number(gift.number);
-}
+[[nodiscard]] QString UniqueGiftName(const UniqueGift &gift);
+
+[[nodiscard]] CreditsAmount UniqueGiftResaleStars(const UniqueGift &gift);
+[[nodiscard]] CreditsAmount UniqueGiftResaleTon(const UniqueGift &gift);
+[[nodiscard]] CreditsAmount UniqueGiftResaleAsked(const UniqueGift &gift);
+
+[[nodiscard]] TextWithEntities FormatGiftResaleStars(const UniqueGift &gift);
+[[nodiscard]] TextWithEntities FormatGiftResaleTon(const UniqueGift &gift);
+[[nodiscard]] TextWithEntities FormatGiftResaleAsked(const UniqueGift &gift);
 
 struct StarGift {
 	uint64 id = 0;
@@ -68,15 +77,20 @@ struct StarGift {
 	int64 starsToUpgrade = 0;
 	int64 starsResellMin = 0;
 	not_null<DocumentData*> document;
+	PeerData *releasedBy = nullptr;
 	QString resellTitle;
 	int resellCount = 0;
 	int limitedLeft = 0;
 	int limitedCount = 0;
+	int perUserTotal = 0;
+	int perUserRemains = 0;
 	TimeId firstSaleDate = 0;
 	TimeId lastSaleDate = 0;
-	bool upgradable = false;
-	bool birthday = false;
-	bool soldOut = false;
+	bool resellTonOnly : 1 = false;
+	bool requirePremium : 1 = false;
+	bool upgradable : 1 = false;
+	bool birthday : 1 = false;
+	bool soldOut : 1 = false;
 
 	friend inline bool operator==(
 		const StarGift &,
@@ -121,8 +135,11 @@ public:
 	}
 
 	friend inline bool operator==(
-		const SavedStarGiftId &a,
-		const SavedStarGiftId &b) = default;
+		const SavedStarGiftId &,
+		const SavedStarGiftId &) = default;
+	friend inline auto operator<=>(
+		const SavedStarGiftId &,
+		const SavedStarGiftId &) = default;
 
 private:
 	PeerData *peer = nullptr;
@@ -133,6 +150,7 @@ private:
 struct SavedStarGift {
 	StarGift info;
 	SavedStarGiftId manageId;
+	std::vector<int> collectionIds;
 	TextWithEntities message;
 	int64 starsConverted = 0;
 	int64 starsUpgradedBySender = 0;
@@ -145,4 +163,12 @@ struct SavedStarGift {
 	bool mine = false;
 };
 
+struct GiftCollection {
+	int id = 0;
+	int count = 0;
+	QString title;
+	DocumentData *icon = nullptr;
+	uint64 hash = 0;
+};
+
 } // namespace Data
