@@ -80,6 +80,7 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 		struct State {
 			TopBarSuggestionContent *content = nullptr;
 			Ui::SlideWrap<Ui::RpWidget> *wrap = nullptr;
+			rpl::variable<int> leftPadding;
 			rpl::variable<Toggle> desiredWrapToggle;
 			rpl::variable<bool> outerWrapToggle;
 			rpl::lifetime birthdayLifetime;
@@ -92,6 +93,8 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 
 		const auto state = lifetime.make_state<State>();
 		state->outerWrapToggle = rpl::duplicate(outerWrapToggleValue);
+		state->leftPadding = rpl::variable<int>(
+			rpl::single(st::dialogsTopBarLeftPadding));
 		const auto ensureWrap = [=] {
 			if (!state->content) {
 				state->content = Ui::CreateChild<TopBarSuggestionContent>(
@@ -112,6 +115,15 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 			}
 		};
 
+		const auto setLeftPaddingRelativeTo = [=](
+				not_null<TopBarSuggestionContent*> content,
+				not_null<Ui::RpWidget*> relativeTo) {
+			content->setLeftPadding(state->leftPadding.value(
+				) | rpl::map([w = relativeTo->width()](int padding) {
+					return w + padding * 2;
+				}));
+		};
+
 		const auto processCurrentSuggestion = [=](auto repeat) -> void {
 			state->birthdayLifetime.destroy();
 			state->premiumLifetime.destroy();
@@ -126,7 +138,7 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 			const auto promo = &session->promoSuggestions();
 			if (const auto custom = promo->custom()) {
 				content->setRightIcon(RightIcon::Close);
-				content->setLeftPadding(0);
+				content->setLeftPadding(state->leftPadding.value());
 				content->setClickedCallback([=] {
 					const auto controller = FindSessionController(parent);
 					UrlClickHandler::Open(
@@ -150,7 +162,7 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 			} else if (session->premiumCanBuy()
 				&& promo->current(kSugPremiumGrace.utf8())) {
 				content->setRightIcon(RightIcon::Close);
-				content->setLeftPadding(0);
+				content->setLeftPadding(state->leftPadding.value());
 				content->setClickedCallback([=] {
 					const auto controller = FindSessionController(parent);
 					UrlClickHandler::Open(
@@ -187,7 +199,7 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 						return;
 					}
 					content->setRightIcon(RightIcon::Close);
-					content->setLeftPadding(0);
+					content->setLeftPadding(state->leftPadding.value());
 					content->setClickedCallback([=] {
 						const auto controller = FindSessionController(parent);
 						controller->uiShow()->show(Box(
@@ -298,8 +310,6 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 							tr::now,
 							TextWithEntities::Simple);
 					content->setContent(std::move(title), std::move(text));
-					const auto leftPadding
-						= st::defaultDialogRow.padding.left();
 					state->giftsLifetime.destroy();
 					if (!isSingle) {
 						struct UserViews {
@@ -348,11 +358,16 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 									s->inRow,
 									st,
 									3);
-								content->setLeftPadding(leftPadding
-									+ (users.size() * st.size - st.shift));
+								const auto v = int(users.size() * st.size
+									- st.shift);
+								content->setLeftPadding(
+									state->leftPadding.value(
+									) | rpl::map([v](int padding) {
+										return padding * 2 + v;
+									}));
 							}
 							p.drawImage(
-								leftPadding,
+								state->leftPadding.current(),
 								(widget->height()
 									- (s->userpics.height()
 										/ style::DevicePixelRatio())) / 2,
@@ -368,15 +383,17 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 									st::uploadUserpicButton));
 						const auto fake = ptr->get();
 						fake->setAttribute(Qt::WA_TransparentForMouseEvents);
-						content->sizeValue() | rpl::filter_size(
-						) | rpl::start_with_next([=](const QSize &s) {
+						rpl::combine(
+							state->leftPadding.value(),
+							content->sizeValue() | rpl::filter_size()
+						) | rpl::start_with_next([=](int p, const QSize &s) {
 							fake->raise();
 							fake->show();
 							fake->moveToLeft(
-								leftPadding,
+								p,
 								(s.height() - fake->height()) / 2);
 						}, fake->lifetime());
-						content->setLeftPadding(fake->width() + leftPadding);
+						setLeftPaddingRelativeTo(content, fake);
 					}
 
 					state->desiredWrapToggle.force_assign(
@@ -386,7 +403,7 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 			} else if (promo->current(kSugSetBirthday.utf8())
 				&& !Data::IsBirthdayToday(session->user()->birthday())) {
 				content->setRightIcon(RightIcon::Close);
-				content->setLeftPadding(0);
+				content->setLeftPadding(state->leftPadding.value());
 				content->setClickedCallback([=] {
 					const auto controller = FindSessionController(parent);
 					Core::App().openInternalUrl(
@@ -461,7 +478,7 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 				};
 				if (isPremiumAnnual || isPremiumRestore || isPremiumUpgrade) {
 					content->setRightIcon(RightIcon::Arrow);
-					content->setLeftPadding(0);
+					content->setLeftPadding(state->leftPadding.value());
 					const auto api = &session->api().premium();
 					api->statusTextValue() | rpl::start_with_next([=] {
 						for (const auto &o : api->subscriptionOptions()) {
@@ -485,16 +502,17 @@ rpl::producer<Ui::SlideWrap<Ui::RpWidget>*> TopBarSuggestionValue(
 					&controller->window(),
 					Ui::UserpicButton::Role::ChoosePhoto,
 					st::uploadUserpicButton);
-				const auto leftPadding = st::defaultDialogRow.padding.left();
-				content->sizeValue() | rpl::filter_size(
-				) | rpl::start_with_next([=](const QSize &s) {
+				rpl::combine(
+					state->leftPadding.value(),
+					content->sizeValue() | rpl::filter_size()
+				) | rpl::start_with_next([=](int padding, const QSize &s) {
 					upload->raise();
 					upload->show();
 					upload->moveToLeft(
-						leftPadding,
+						padding,
 						(s.height() - upload->height()) / 2);
 				}, content->lifetime());
-				content->setLeftPadding(upload->width() + leftPadding);
+				setLeftPaddingRelativeTo(content, upload);
 				upload->chosenImages() | rpl::start_with_next([=](
 						Ui::UserpicButton::ChosenImage &&chosen) {
 					if (chosen.type == Ui::UserpicButton::ChosenType::Set) {
