@@ -7,6 +7,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
+#include "dialogs/ui/dialogs_quick_action_context.h"
 #include "ui/cached_round_corners.h"
 
 namespace style {
@@ -18,9 +19,6 @@ namespace st {
 extern const style::DialogRow &defaultDialogRow;
 } // namespace st
 
-namespace Ui {
-} // namespace Ui
-
 namespace Data {
 class Forum;
 class Folder;
@@ -57,6 +55,7 @@ struct TopicJumpCache {
 struct PaintContext {
 	RightButton *rightButton = nullptr;
 	std::vector<QImage*> *chatsFilterTags = nullptr;
+	QuickActionContext *quickActionContext = nullptr;
 	not_null<const style::DialogRow*> st;
 	TopicJumpCache *topicJumpCache = nullptr;
 	Data::Folder *folder = nullptr;
@@ -111,4 +110,6 @@ void PaintCollapsedRow(
 	int unread,
 	const PaintContext &context);
 
+int PaintRightButton(QPainter &p, const PaintContext &context);
+
 } // namespace Dialogs::Ui
