@@ -11,6 +11,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "api/api_sending.h"
 #include "apiwrap.h"
 #include "base/random.h"
+#include "ui/effects/premium_stars.h"
 #include "boxes/premium_preview_box.h"
 #include "chat_helpers/stickers_lottie.h"
 #include "core/click_handler_types.h"
@@ -18,6 +19,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "countries/countries_instance.h"
 #include "data/business/data_business_common.h"
 #include "data/stickers/data_custom_emoji.h"
+#include "data/data_channel.h"
 #include "data/data_document.h"
 #include "data/data_session.h"
 #include "data/data_user.h"
@@ -61,6 +63,7 @@ public:
 	enum class Type {
 		PremiumRequired,
 		StarsCharged,
+		FreeDirect,
 	};
 
 	EmptyChatLockedBox(not_null<Element*> parent, Type type);
@@ -73,7 +76,7 @@ public:
 	TextWithEntities subtitle() override;
 	int buttonSkip() override;
 	rpl::producer<QString> button() override;
-	bool buttonMinistars() override;
+	std::optional<Ui::Premium::MiniStarsType> buttonMinistars() override;
 	void draw(
 		Painter &p,
 		const PaintContext &context,
@@ -401,7 +404,9 @@ EmptyChatLockedBox::EmptyChatLockedBox(not_null<Element*> parent, Type type)
 EmptyChatLockedBox::~EmptyChatLockedBox() = default;
 
 int EmptyChatLockedBox::width() {
-	return st::premiumRequiredWidth;
+	return (_type == Type::PremiumRequired)
+		? st::premiumRequiredWidth
+		: st::starsPerMessageWidth;
 }
 
 int EmptyChatLockedBox::top() {
@@ -421,13 +426,16 @@ int EmptyChatLockedBox::buttonSkip() {
 }
 
 rpl::producer<QString> EmptyChatLockedBox::button() {
-	return (_type == Type::PremiumRequired)
+	return (_type == Type::FreeDirect)
+		? nullptr
+		: (_type == Type::PremiumRequired)
 		? tr::lng_send_non_premium_go()
 		: tr::lng_send_charges_stars_go();
 }
 
-bool EmptyChatLockedBox::buttonMinistars() {
-	return true;
+auto EmptyChatLockedBox::buttonMinistars()
+-> std::optional<Ui::Premium::MiniStarsType> {
+	return Ui::Premium::MiniStarsType::SlowStars;
 }
 
 TextWithEntities EmptyChatLockedBox::subtitle() {
@@ -456,7 +464,9 @@ void EmptyChatLockedBox::draw(
 	p.setBrush(context.st->msgServiceBg()); // ?
 	p.setPen(Qt::NoPen);
 	p.drawEllipse(geometry);
-	st::premiumRequiredIcon.paintInCenter(p, geometry);
+	(_type == Type::PremiumRequired
+		? st::premiumRequiredIcon
+		: st::directMessagesIcon).paintInCenter(p, geometry);
 }
 
 void EmptyChatLockedBox::stickerClearLoopPlayed() {
@@ -512,6 +522,9 @@ bool AboutView::refresh() {
 		return true;
 	}
 	const auto user = _history->peer->asUser();
+	const auto monoforum = _history->peer->isMonoforum()
+		? _history->peer->asChannel()
+		: nullptr;
 	const auto info = user ? user->botInfo.get() : nullptr;
 	if (!info) {
 		if (user
@@ -539,6 +552,14 @@ bool AboutView::refresh() {
 				makeIntro(user);
 			}
 			return true;
+		} else if (monoforum && _history->isDisplayedEmpty()) {
+			if (_item) {
+				return false;
+			}
+			setItem(
+				makeStarsPerMessage(monoforum->starsPerMessageChecked()),
+				nullptr);
+			return true;
 		}
 		if (_item) {
 			setItem({}, nullptr);
@@ -600,6 +621,8 @@ void AboutView::make(Data::ChatIntro data, bool preview) {
 	const auto sendIntroSticker = [=](not_null<DocumentData*> sticker) {
 		_sendIntroSticker.fire_copy(sticker);
 	};
+	owned->data()->setCustomServiceLink(
+		std::make_shared<LambdaClickHandler>(handler));
 	owned->overrideMedia(std::make_unique<HistoryView::MediaGeneric>(
 		owned.get(),
 		GenerateChatIntro(
@@ -610,7 +633,6 @@ void AboutView::make(Data::ChatIntro data, bool preview) {
 			sendIntroSticker),
 		HistoryView::MediaGenericDescriptor{
 			.maxWidth = st::chatIntroWidth,
-			.serviceLink = std::make_shared<LambdaClickHandler>(handler),
 			.service = true,
 			.hideServiceText = preview || text.isEmpty(),
 		}));
@@ -813,28 +835,46 @@ AdminLog::OwnedItem AboutView::makePremiumRequired() {
 }
 
 AdminLog::OwnedItem AboutView::makeStarsPerMessage(int stars) {
+	auto name = Ui::Text::Bold(_history->peer->shortName());
+	auto cost = Ui::Text::IconEmoji(
+		&st::starIconEmoji
+	).append(Ui::Text::Bold(Lang::FormatCountDecimal(stars)));
 	const auto item = _history->makeMessage({
 		.id = _history->nextNonHistoryEntryId(),
 		.flags = (MessageFlag::FakeAboutView
 			| MessageFlag::FakeHistoryItem
 			| MessageFlag::Local),
 		.from = _history->peer->id,
-	}, PreparedServiceText{ tr::lng_send_charges_stars_text(
-		tr::now,
-		lt_user,
-		Ui::Text::Bold(_history->peer->shortName()),
-		lt_amount,
-		Ui::Text::IconEmoji(
-			&st::starIconEmoji
-		).append(Ui::Text::Bold(Lang::FormatCountDecimal(stars))),
-		Ui::Text::RichLangValue),
+	}, PreparedServiceText{ !_history->peer->isMonoforum()
+		? tr::lng_send_charges_stars_text(
+			tr::now,
+			lt_user,
+			std::move(name),
+			lt_amount,
+			std::move(cost),
+			Ui::Text::RichLangValue)
+		: stars
+		? tr::lng_send_charges_stars_channel(
+			tr::now,
+			lt_channel,
+			std::move(name),
+			lt_amount,
+			std::move(cost),
+			Ui::Text::RichLangValue)
+		: tr::lng_send_free_channel(
+			tr::now,
+			lt_channel,
+			std::move(name),
+			Ui::Text::RichLangValue),
 	});
 	auto result = AdminLog::OwnedItem(_delegate, item);
 	result->overrideMedia(std::make_unique<ServiceBox>(
 		result.get(),
 		std::make_unique<EmptyChatLockedBox>(
 			result.get(),
-			EmptyChatLockedBox::Type::StarsCharged)));
+			(stars
+				? EmptyChatLockedBox::Type::StarsCharged
+				: EmptyChatLockedBox::Type::FreeDirect))));
 	return result;
 }
 
