@@ -334,9 +334,11 @@ void TodoList::updateTasks(bool skipAnimations) {
 ClickHandlerPtr TodoList::createTaskClickHandler(
 		const Task &task) {
 	const auto id = task.id;
-	return std::make_shared<LambdaClickHandler>(crl::guard(this, [=] {
+	auto result = std::make_shared<LambdaClickHandler>(crl::guard(this, [=] {
 		toggleCompletion(id);
 	}));
+	result->setProperty(kTodoListItemIdProperty, id);
+	return result;
 }
 
 void TodoList::startToggleAnimation(Task &task) {
@@ -375,11 +377,24 @@ void TodoList::toggleCompletion(int id) {
 	if (i == end(_tasks)) {
 		return;
 	}
+
 	const auto selected = (i->completionDate != 0);
 	i->completionDate = selected ? TimeId() : base::unixtime::now();
 	if (!selected) {
 		i->setCompletedBy(_parent->history()->session().user());
 	}
+
+	const auto parentMedia = _parent->data()->media();
+	const auto baseList = parentMedia ? parentMedia->todolist() : nullptr;
+	if (baseList) {
+		const auto j = ranges::find(baseList->items, id, &TodoListItem::id);
+		if (j != end(baseList->items)) {
+			j->completionDate = i->completionDate;
+			j->completedBy = i->completedBy;
+		}
+		history()->owner().updateDependentMessages(_parent->data());
+	}
+
 	startToggleAnimation(*i);
 	repaint();
 
@@ -467,6 +482,7 @@ void TodoList::draw(Painter &p, const PaintContext &context) const {
 			paintw,
 			width(),
 			context);
+		appendTaskHighlight(task.id, tshift, height, context);
 		if (was) {
 			heavy = true;
 		} else if (!task.userpic.null()) {
@@ -561,6 +577,33 @@ int TodoList::paintTask(
 	return height;
 }
 
+void TodoList::appendTaskHighlight(
+		int id,
+		int top,
+		int height,
+		const PaintContext &context) const {
+	if (context.highlight.todoItemId != id
+		|| context.highlight.collapsion <= 0.) {
+		return;
+	}
+	const auto to = context.highlightInterpolateTo;
+	const auto toProgress = (1. - context.highlight.collapsion);
+	if (toProgress >= 1.) {
+		context.highlightPathCache->addRect(to);
+	} else if (toProgress <= 0.) {
+		context.highlightPathCache->addRect(0, top, width(), height);
+	} else {
+		const auto lerp = [=](int from, int to) {
+			return from + (to - from) * toProgress;
+		};
+		context.highlightPathCache->addRect(
+			lerp(0, to.x()),
+			lerp(top, to.y()),
+			lerp(width(), to.width()),
+			lerp(height, to.height()));
+	}
+}
+
 void TodoList::paintRadio(
 		Painter &p,
 		const Task &task,
