@@ -12,13 +12,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 namespace Info::ChannelEarn {
 
 [[nodiscard]] QString MajorPart(Data::EarnInt value);
+[[nodiscard]] QString MajorPart(CreditsAmount value);
 [[nodiscard]] QString MinorPart(Data::EarnInt value);
+[[nodiscard]] QString MinorPart(CreditsAmount value);
 [[nodiscard]] QString ToUsd(
 	Data::EarnInt value,
 	float64 rate,
 	int afterFloat);
 [[nodiscard]] QString ToUsd(
-	StarsAmount value,
+	CreditsAmount value,
 	float64 rate,
 	int afterFloat);
 
