@@ -47,11 +47,6 @@ namespace Info::Downloads {
 struct Tag;
 } // namespace Info::Downloads
 
-namespace Info::Stories {
-struct Tag;
-enum class Tab;
-} // namespace Info::Stories
-
 namespace Info::Statistics {
 struct Tag;
 } // namespace Info::Statistics
@@ -65,6 +60,14 @@ namespace Info::GlobalMedia {
 struct Tag;
 } // namespace Info::GlobalMedia
 
+namespace Info::PeerGifts {
+struct Tag;
+} // namespace Info::PeerGifts
+
+namespace Info::Stories {
+struct Tag;
+} // namespace Info::Stories
+
 namespace Info {
 
 class ContentMemento;
@@ -212,6 +215,7 @@ public:
 		Data::ForumTopic *topic,
 		Data::SavedSublist *sublist,
 		PeerId migratedPeerId);
+	explicit ContentMemento(PeerGifts::Tag gifts);
 	explicit ContentMemento(Settings::Tag settings);
 	explicit ContentMemento(Downloads::Tag downloads);
 	explicit ContentMemento(Stories::Tag stories);
@@ -226,65 +230,74 @@ public:
 		std::shared_ptr<Api::WhoReadList> whoReadIds,
 		FullMsgId contextId,
 		Data::ReactionId selected);
+	virtual ~ContentMemento() = default;
 
-	virtual object_ptr<ContentWidget> createWidget(
+	[[nodiscard]] virtual object_ptr<ContentWidget> createWidget(
 		QWidget *parent,
 		not_null<Controller*> controller,
 		const QRect &geometry) = 0;
 
-	PeerData *peer() const {
+	[[nodiscard]] PeerData *peer() const {
 		return _peer;
 	}
-	PeerId migratedPeerId() const {
+	[[nodiscard]] PeerId migratedPeerId() const {
 		return _migratedPeerId;
 	}
-	Data::ForumTopic *topic() const {
+	[[nodiscard]] Data::ForumTopic *topic() const {
 		return _topic;
 	}
-	Data::SavedSublist *sublist() const {
+	[[nodiscard]] Data::SavedSublist *sublist() const {
 		return _sublist;
 	}
-	UserData *settingsSelf() const {
+	[[nodiscard]] UserData *settingsSelf() const {
 		return _settingsSelf;
 	}
-	PeerData *storiesPeer() const {
+	[[nodiscard]] PeerData *storiesPeer() const {
 		return _storiesPeer;
 	}
-	Stories::Tab storiesTab() const {
-		return _storiesTab;
+	[[nodiscard]] int storiesAlbumId() const {
+		return _storiesAlbumId;
 	}
-	Statistics::Tag statisticsTag() const {
+	[[nodiscard]] int storiesAddToAlbumId() const {
+		return _storiesAddToAlbumId;
+	}
+	[[nodiscard]] PeerData *giftsPeer() const {
+		return _giftsPeer;
+	}
+	[[nodiscard]] int giftsCollectionId() const {
+		return _giftsCollectionId;
+	}
+	[[nodiscard]] Statistics::Tag statisticsTag() const {
 		return _statisticsTag;
 	}
-	PeerData *starrefPeer() const {
+	[[nodiscard]] PeerData *starrefPeer() const {
 		return _starrefPeer;
 	}
-	BotStarRef::Type starrefType() const {
+	[[nodiscard]] BotStarRef::Type starrefType() const {
 		return _starrefType;
 	}
-	PollData *poll() const {
+	[[nodiscard]] PollData *poll() const {
 		return _poll;
 	}
-	FullMsgId pollContextId() const {
+	[[nodiscard]] FullMsgId pollContextId() const {
 		return _poll ? _pollReactionsContextId : FullMsgId();
 	}
-	std::shared_ptr<Api::WhoReadList> reactionsWhoReadIds() const {
+	[[nodiscard]] auto reactionsWhoReadIds() const
+	-> std::shared_ptr<Api::WhoReadList> {
 		return _reactionsWhoReadIds;
 	}
-	Data::ReactionId reactionsSelected() const {
+	[[nodiscard]] Data::ReactionId reactionsSelected() const {
 		return _reactionsSelected;
 	}
-	FullMsgId reactionsContextId() const {
+	[[nodiscard]] FullMsgId reactionsContextId() const {
 		return _reactionsWhoReadIds ? _pollReactionsContextId : FullMsgId();
 	}
-	UserData *globalMediaSelf() const {
+	[[nodiscard]] UserData *globalMediaSelf() const {
 		return _globalMediaSelf;
 	}
-	Key key() const;
+	[[nodiscard]] Key key() const;
 
-	virtual Section section() const = 0;
-
-	virtual ~ContentMemento() = default;
+	[[nodiscard]] virtual Section section() const = 0;
 
 	void setScrollTop(int scrollTop) {
 		_scrollTop = scrollTop;
@@ -295,19 +308,19 @@ public:
 	void setSearchFieldQuery(const QString &query) {
 		_searchFieldQuery = query;
 	}
-	QString searchFieldQuery() const {
+	[[nodiscard]] QString searchFieldQuery() const {
 		return _searchFieldQuery;
 	}
 	void setSearchEnabledByContent(bool enabled) {
 		_searchEnabledByContent = enabled;
 	}
-	bool searchEnabledByContent() const {
+	[[nodiscard]] bool searchEnabledByContent() const {
 		return _searchEnabledByContent;
 	}
 	void setSearchStartsFocused(bool focused) {
 		_searchStartsFocused = focused;
 	}
-	bool searchStartsFocused() const {
+	[[nodiscard]] bool searchStartsFocused() const {
 		return _searchStartsFocused;
 	}
 
@@ -318,7 +331,10 @@ private:
 	Data::SavedSublist *_sublist = nullptr;
 	UserData * const _settingsSelf = nullptr;
 	PeerData * const _storiesPeer = nullptr;
-	Stories::Tab _storiesTab = {};
+	int _storiesAlbumId = 0;
+	int _storiesAddToAlbumId = 0;
+	PeerData * const _giftsPeer = nullptr;
+	int _giftsCollectionId = 0;
 	Statistics::Tag _statisticsTag;
 	PeerData * const _starrefPeer = nullptr;
 	BotStarRef::Type _starrefType = {};
