@@ -47,6 +47,16 @@ class SessionController;
 
 namespace Info::PeerGifts {
 
+struct Tag {
+	explicit Tag(not_null<PeerData*> peer, int collectionId = 0)
+	: peer(peer)
+	, collectionId(collectionId) {
+	}
+
+	not_null<PeerData*> peer;
+	int collectionId = 0;
+};
+
 struct GiftTypePremium {
 	int64 cost = 0;
 	QString currency;
@@ -65,6 +75,7 @@ struct GiftTypeStars {
 	PeerData *from = nullptr;
 	TimeId date = 0;
 	bool pinnedSelection : 1 = false;
+	bool forceTon : 1 = false;
 	bool userpic : 1 = false;
 	bool pinned : 1 = false;
 	bool hidden : 1 = false;
@@ -115,6 +126,7 @@ class GiftButtonDelegate {
 public:
 	[[nodiscard]] virtual TextWithEntities star() = 0;
 	[[nodiscard]] virtual TextWithEntities monostar() = 0;
+	[[nodiscard]] virtual TextWithEntities monoton() = 0;
 	[[nodiscard]] virtual TextWithEntities ministar() = 0;
 	[[nodiscard]] virtual Ui::Text::MarkedContext textContext() = 0;
 	[[nodiscard]] virtual QSize buttonSize() = 0;
@@ -139,7 +151,9 @@ public:
 	void setDescriptor(const GiftDescriptor &descriptor, Mode mode);
 	void setGeometry(QRect inner, QMargins extend);
 
-	void toggleSelected(bool selected);
+	void toggleSelected(
+		bool selected,
+		anim::type animated = anim::type::normal);
 
 	[[nodiscard]] rpl::producer<QPoint> contextMenuRequests() const {
 		return _contextMenuRequests.events();
@@ -157,7 +171,6 @@ private:
 		int height);
 
 	void setDocument(not_null<DocumentData*> document);
-	[[nodiscard]] bool documentResolved() const;
 	[[nodiscard]] QMargins currentExtend() const;
 
 	void unsubscribe();
@@ -171,6 +184,7 @@ private:
 	Ui::Text::String _byStars;
 	std::shared_ptr<Ui::DynamicImage> _userpic;
 	QImage _uniqueBackgroundCache;
+	QImage _tonIcon;
 	std::unique_ptr<Ui::Text::CustomEmoji> _uniquePatternEmoji;
 	base::flat_map<float64, QImage> _uniquePatternCache;
 	std::optional<Ui::Premium::ColoredMiniStars> _stars;
@@ -184,8 +198,12 @@ private:
 	QRect _button;
 	QMargins _extend;
 
+	DocumentData *_resolvedDocument = nullptr;
+
 	std::unique_ptr<HistoryView::StickerPlayer> _player;
+	DocumentData *_playerDocument = nullptr;
 	rpl::lifetime _mediaLifetime;
+	rpl::lifetime _documentLifetime;
 
 };
 
@@ -197,6 +215,7 @@ public:
 
 	TextWithEntities star() override;
 	TextWithEntities monostar() override;
+	TextWithEntities monoton() override;
 	TextWithEntities ministar() override;
 	Ui::Text::MarkedContext textContext() override;
 	QSize buttonSize() override;
