@@ -11,6 +11,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "api/api_statistics.h"
 #include "boxes/peer_list_controllers.h"
 #include "boxes/peer_list_widgets.h"
+#include "info/channel_statistics/earn/earn_icons.h"
+#include "info/channel_statistics/earn/earn_format.h"
 #include "chat_helpers/stickers_gift_box_pack.h"
 #include "core/ui_integration.h" // TextContext
 #include "data/data_channel.h"
@@ -32,6 +34,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/effects/toggle_arrow.h"
 #include "ui/painter.h"
 #include "ui/rect.h"
+#include "ui/text/format_values.h"
+#include "ui/text/text_utilities.h"
 #include "ui/vertical_list.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/popup_menu.h"
@@ -39,6 +43,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/wrap/vertical_layout.h"
 #include "styles/style_boxes.h"
 #include "styles/style_color_indices.h"
+#include "styles/style_channel_earn.h"
 #include "styles/style_credits.h"
 #include "styles/style_dialogs.h" // dialogsStoriesFull.
 #include "styles/style_layers.h" // boxRowPadding.
@@ -46,6 +51,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "styles/style_settings.h"
 #include "styles/style_statistics.h"
 #include "styles/style_window.h"
+#include "styles/style_chat.h"
 
 namespace Info::Statistics {
 namespace {
@@ -403,7 +409,7 @@ void PublicForwardsController::appendRow(
 	const auto views = [&] {
 		if (contextId.messageId) {
 			const auto message = peer->owner().message(contextId.messageId);
-			return message ? message->viewsCount() : 0;
+			return message ? std::max(message->viewsCount(), 0) : 0;
 		} else if (const auto &id = contextId.storyId) {
 			const auto story = peer->owner().stories().lookup(id);
 			return story ? (*story)->views() : 0;
@@ -744,6 +750,29 @@ rpl::producer<int> BoostsController::totalBoostsValue() const {
 	return _totalBoosts.value();
 }
 
+struct CreditsRowDescriptionData {
+	uint64 rowId = 0;
+	uint64 bareGiftStickerId = 0;
+};
+
+[[nodiscard]] QString SerializeCreditsRowDescriptionData(
+		const CreditsRowDescriptionData &data) {
+	return QString("%1;%2").arg(data.rowId).arg(data.bareGiftStickerId);
+}
+
+[[nodiscard]] CreditsRowDescriptionData DeserializeCreditsRowDescriptionData(
+		const QString &str) {
+	auto data = CreditsRowDescriptionData();
+	const auto parts = str.split(';');
+	if (parts.size() >= 1) {
+		data.rowId = parts[0].toULongLong();
+	}
+	if (parts.size() >= 2) {
+		data.bareGiftStickerId = parts[1].toULongLong();
+	}
+	return data;
+}
+
 class CreditsRow final : public PeerListRow {
 public:
 	struct Descriptor final {
@@ -758,6 +787,8 @@ public:
 	CreditsRow(not_null<PeerData*> peer, const Descriptor &descriptor);
 	CreditsRow(const Descriptor &descriptor);
 
+	void init();
+
 	[[nodiscard]] const Data::CreditsHistoryEntry &entry() const;
 	[[nodiscard]] const Data::SubscriptionEntry &subscription() const;
 	[[nodiscard]] QString generateName() override;
@@ -789,12 +820,11 @@ public:
 		const style::PeerListItem &st) const override;
 
 private:
-	void init();
-
 	const not_null<Main::Session*> _session;
 	const Data::CreditsHistoryEntry _entry;
 	const Data::SubscriptionEntry _subscription;
 	const Ui::Text::MarkedContext _context;
+
 	const int _rowHeight;
 
 	PaintRoundImageCallback _paintUserpicCallback;
@@ -803,6 +833,7 @@ private:
 
 	Ui::Text::String _description;
 	Ui::Text::String _rightText;
+	Ui::Text::String _rightMinorText;
 
 	std::shared_ptr<Ui::DynamicImage> _descriptionThumbnail;
 	QImage _descriptionThumbnailCache;
@@ -835,7 +866,6 @@ CreditsRow::CreditsRow(
 			st::boostsListBox.item,
 			_subscription.subscription.credits);
 	}
-	init();
 }
 
 CreditsRow::CreditsRow(const Descriptor &descriptor)
@@ -845,7 +875,6 @@ CreditsRow::CreditsRow(const Descriptor &descriptor)
 , _subscription(descriptor.subscription)
 , _context(descriptor.context)
 , _rowHeight(descriptor.rowHeight) {
-	init();
 }
 
 void CreditsRow::init() {
@@ -859,6 +888,8 @@ void CreditsRow::init() {
 			lt_count,
 			_entry.paidMessagesCount)
 		: ((!_entry.subscriptionUntil.isNull() && !isSpecial)
+			|| (_entry.giftUpgraded && !isSpecial)
+			|| (_entry.giftResale && !isSpecial)
 			|| _entry.title.isEmpty())
 		? name
 		: _entry.title;
@@ -883,6 +914,8 @@ void CreditsRow::init() {
 		: (_entry.peerType
 			== Data::CreditsHistoryEntry::PeerType::PremiumBot)
 		? tr::lng_credits_box_history_entry_via_premium_bot(tr::now)
+		: (_entry.peerType == Data::CreditsHistoryEntry::PeerType::Fragment)
+		? tr::lng_credits_box_history_entry_fragment(tr::now)
 		: (_entry.gift && isSpecial)
 		? tr::lng_credits_box_history_entry_anonymous(tr::now)
 		: (_name == name)
@@ -901,6 +934,19 @@ void CreditsRow::init() {
 				langDayOfMonthFull(_subscription.until.date())));
 		_description.setText(st::defaultTextStyle, _subscription.title);
 	}
+	if (_entry.bareGiftStickerId) {
+		_description.setMarkedText(
+			st::defaultTextStyle,
+			Ui::Text::SingleCustomEmoji(
+				SerializeCreditsRowDescriptionData({
+					PeerListRow::id(),
+					_entry.bareGiftStickerId,
+				}))
+			.append(' ')
+			.append(description),
+			kMarkupTextOptions,
+			_context);
+	}
 	const auto descriptionPhotoId = (!_entry.subscriptionUntil.isNull())
 		? _entry.photoId
 		: _subscription.photoId;
@@ -918,26 +964,45 @@ void CreditsRow::init() {
 			}
 		});
 	}
-	auto &manager = _session->data().customEmojiManager();
 	if (_entry) {
 		constexpr auto kMinus = QChar(0x2212);
+		const auto isCurrency = _entry.credits.ton();
 		_rightText.setMarkedText(
-			st::semiboldTextStyle,
+			isCurrency
+				? st::channelEarnHistoryMajorLabel.style
+				: st::creditsHistoryRowRightStyle,
 			TextWithEntities()
 				.append(_entry.in ? QChar('+') : kMinus)
-				.append(Lang::FormatStarsAmountDecimal(_entry.credits.abs()))
+				.append(isCurrency
+					? Info::ChannelEarn::MajorPart(_entry.credits)
+					: Lang::FormatCreditsAmountDecimal(_entry.credits.abs()))
 				.append(QChar(' '))
-				.append(manager.creditsEmoji()),
+				.append(isCurrency
+					? TextWithEntities()
+					: Ui::MakeCreditsIconEntity()),
 			kMarkupTextOptions,
 			_context);
+		if (isCurrency) {
+			_rightMinorText.setMarkedText(
+				st::channelEarnHistoryMinorLabel.style,
+				TextWithEntities()
+					.append(Info::ChannelEarn::MinorPart(_entry.credits))
+					.append(QChar(' '))
+					.append(
+						Ui::Text::SingleCustomEmoji(_entry.in
+							? u"ton:in"_q
+							: u"ton:out"_q)),
+				kMarkupTextOptions,
+				_context);
+		}
 	}
 	if (!_paintUserpicCallback) {
-		_paintUserpicCallback = _entry.stargift
+		_paintUserpicCallback = /*_entry.stargift
 			? Ui::GenerateGiftStickerUserpicCallback(
 				_session,
 				_entry.bareGiftStickerId,
 				_context.repaint)
-			: !isSpecial
+			: */!isSpecial
 			? PeerListRow::generatePaintUserpicCallback(false)
 			: Ui::GenerateCreditsPaintUserpicCallback(_entry);
 	}
@@ -984,7 +1049,9 @@ QSize CreditsRow::rightActionSize() const {
 		return QSize(maxWidth + st::boxRowPadding.right(), _rowHeight);
 	} else if (_subscription || _entry) {
 		return QSize(
-			_rightText.maxWidth() + st::boxRowPadding.right() / 2,
+			_rightText.maxWidth()
+				+ _rightMinorText.maxWidth()
+				+ st::boxRowPadding.right() / 2,
 			_rowHeight);
 	} else if (!_entry && !_subscription) {
 		return QSize();
@@ -1007,7 +1074,6 @@ void CreditsRow::rightActionPaint(
 		int outerWidth,
 		bool selected,
 		bool actionSelected) {
-	const auto &font = _rightText.style()->font;
 	const auto rightSkip = st::boxRowPadding.right();
 	if (_rightLabel) {
 		return _rightLabel->draw(p, x, y, _rowHeight);
@@ -1038,16 +1104,21 @@ void CreditsRow::rightActionPaint(
 		p.drawTextRight(rightSkip, y - statusFont->height / 2, outerWidth, t);
 		return;
 	}
-	y += _rowHeight / 2;
 	p.setPen(_entry.pending
 		? st::creditsStroke
 		: _entry.in
 		? st::boxTextFgGood
 		: st::menuIconAttentionColor);
+	const auto xMinor = outerWidth - _rightMinorText.maxWidth() - rightSkip;
+	_rightMinorText.draw(p, Ui::Text::PaintContext{
+		.position = QPoint(xMinor, y + st::creditsHistoryRowRightMinorTop),
+		.outerWidth = outerWidth,
+		.availableWidth = outerWidth,
+	});
 	_rightText.draw(p, Ui::Text::PaintContext{
 		.position = QPoint(
-			outerWidth - _rightText.maxWidth() - rightSkip,
-			y - font->height / 2),
+			xMinor - _rightText.maxWidth(),
+			y + st::creditsHistoryRowRightTop),
 		.outerWidth = outerWidth,
 		.availableWidth = outerWidth,
 	});
@@ -1076,7 +1147,7 @@ void CreditsRow::paintStatusText(
 		available -= thumbnailSpace;
 	}
 	_description.draw(p, {
-		.position = QPoint(x, y - _description.minHeight()),
+		.position = QPoint(x, y - st::creditsHistoryRowDescriptionSkip),
 		.outerWidth = outer,
 		.availableWidth = available,
 		.elisionLines = 1,
@@ -1116,6 +1187,8 @@ private:
 	Data::CreditsStatusSlice::OffsetToken _apiToken;
 	Ui::Text::MarkedContext _context;
 
+	base::flat_map<PeerListRowId, not_null<PeerListRow*>> _rowsById;
+
 	rpl::variable<bool> _allLoaded = false;
 	bool _requesting = false;
 
@@ -1127,7 +1200,46 @@ CreditsController::CreditsController(CreditsDescriptor d)
 , _entryClickedCallback(std::move(d.entryClickedCallback))
 , _api(d.peer, d.in, d.out)
 , _firstSlice(std::move(d.firstSlice))
-, _context(Core::TextContext({ .session = _session })) {
+, _context([&]() -> Ui::Text::MarkedContext {
+	const auto height = st::creditsHistoryRowRightStyle.font->height
+		- st::lineWidth;
+	auto customEmojiFactory = [=](
+			QStringView data,
+			const Ui::Text::MarkedContext &context
+		) -> std::unique_ptr<Ui::Text::CustomEmoji> {
+		if (data == Ui::kCreditsCurrency) {
+			return std::make_unique<Ui::Text::ShiftedEmoji>(
+				Ui::MakeCreditsIconEmoji(height, 1),
+				QPoint(-st::lineWidth, st::lineWidth));
+		}
+		if (data.startsWith(u"ton"_q)) {
+			const auto in = data.split(u":"_q)[1].startsWith(u"in"_q);
+			return std::make_unique<Ui::Text::ShiftedEmoji>(
+				std::make_unique<Ui::Text::StaticCustomEmoji>(
+					Ui::Earn::IconCurrencyColored(
+						st::tonFieldIconSize,
+						in
+							? st::boxTextFgGood->c
+							: st::menuIconAttentionColor->c),
+					data.toString()),
+				QPoint(0, st::lineWidth));
+		}
+		const auto desc = DeserializeCreditsRowDescriptionData(
+			data.toString());
+		if (!desc.rowId || !desc.bareGiftStickerId) {
+			return nullptr;
+		}
+		const auto it = _rowsById.find(desc.rowId);
+		if (it != _rowsById.end()) {
+			const auto row = it->second;
+			return _session->data().customEmojiManager().create(
+				desc.bareGiftStickerId,
+				[=]{ delegate()->peerListUpdateRow(row); });
+		}
+		return nullptr;
+	};
+	return { .customEmojiFactory = std::move(customEmojiFactory) };
+}()) {
 	PeerListController::setStyleOverrides(&st::creditsHistoryEntriesList);
 }
 
@@ -1178,15 +1290,19 @@ void CreditsController::applySlice(const Data::CreditsStatusSlice &slice) {
 				delegate()->peerListUpdateRow(row);
 			},
 		};
+		auto owned = std::unique_ptr<CreditsRow>(nullptr);
 		if (i.bareActorId) {
 			const auto peer = session().data().peer(PeerId(i.bareActorId));
-			return std::make_unique<CreditsRow>(peer, descriptor);
+			owned = std::make_unique<CreditsRow>(peer, descriptor);
 		} else if (const auto peerId = PeerId(i.barePeerId + s.barePeerId)) {
 			const auto peer = session().data().peer(peerId);
-			return std::make_unique<CreditsRow>(peer, descriptor);
+			owned = std::make_unique<CreditsRow>(peer, descriptor);
 		} else {
-			return std::make_unique<CreditsRow>(descriptor);
+			owned = std::make_unique<CreditsRow>(descriptor);
 		}
+		_rowsById.emplace(owned->id(), owned.get());
+		owned->init();
+		return owned;
 	};
 
 	auto giftPacksRequested = false;
