@@ -39,11 +39,12 @@ QByteArray SessionSettings::serialize() const {
 		+ Serialize::bytearraySize(autoDownload)
 		+ sizeof(qint32) * 11
 		+ (_mutePeriods.size() * sizeof(quint64))
-		+ sizeof(qint32) * 2
-		+ _hiddenPinnedMessages.size() * (sizeof(quint64) * 3)
-		+ sizeof(qint32)
+		+ sizeof(qint32) * 3
 		+ _groupEmojiSectionHidden.size() * sizeof(quint64)
-		+ sizeof(qint32) * 2;
+		+ sizeof(qint32) * 3
+		+ _hiddenPinnedMessages.size() * (sizeof(quint64) * 4)
+		+ sizeof(qint32)
+		+ _verticalSubsectionTabs.size() * sizeof(quint64);
 
 	auto result = QByteArray();
 	result.reserve(size);
@@ -68,32 +69,37 @@ QByteArray SessionSettings::serialize() const {
 			<< qint32(_archiveInMainMenu.current() ? 1 : 0)
 			<< qint32(_skipArchiveInSearch.current() ? 1 : 0)
 			<< qint32(0) // old _mediaLastPlaybackPosition.size());
-			<< qint32(0) // very old _hiddenPinnedMessages.size());
+			<< qint32(0) // very very old _hiddenPinnedMessages.size());
 			<< qint32(_dialogsFiltersEnabled ? 1 : 0)
 			<< qint32(_supportAllSilent ? 1 : 0)
 			<< qint32(_photoEditorHintShowsCount)
-			<< qint32(0) // old _hiddenPinnedMessages.size());
+			<< qint32(0) // very old _hiddenPinnedMessages.size());
 			<< qint32(_mutePeriods.size());
 		for (const auto &period : _mutePeriods) {
 			stream << quint64(period);
 		}
 		stream
 			<< qint32(0) // old _skipPremiumStickersSet
-			<< qint32(_hiddenPinnedMessages.size());
-		for (const auto &[key, value] : _hiddenPinnedMessages) {
-			stream
-				<< SerializePeerId(key.peerId)
-				<< qint64(key.topicRootId.bare)
-				<< qint64(value.bare);
-		}
-		stream
+			<< qint32(0) // old _hiddenPinnedMessages.size());
 			<< qint32(_groupEmojiSectionHidden.size());
 		for (const auto &peerId : _groupEmojiSectionHidden) {
 			stream << SerializePeerId(peerId);
 		}
 		stream
 			<< qint32(_lastNonPremiumLimitDownload)
-			<< qint32(_lastNonPremiumLimitUpload);
+			<< qint32(_lastNonPremiumLimitUpload)
+			<< qint32(_hiddenPinnedMessages.size());
+		for (const auto &[key, value] : _hiddenPinnedMessages) {
+			stream
+				<< SerializePeerId(key.peerId)
+				<< qint64(key.topicRootId.bare)
+				<< SerializePeerId(key.monoforumPeerId)
+				<< qint64(value.bare);
+		}
+		stream << qint32(_verticalSubsectionTabs.size());
+		for (const auto &peerId : _verticalSubsectionTabs) {
+			stream << SerializePeerId(peerId);
+		}
 	}
 
 	Ensures(result.size() == size);
@@ -153,6 +159,7 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 	std::vector<int> appDictionariesEnabled;
 	qint32 appAutoDownloadDictionaries = app.autoDownloadDictionaries() ? 1 : 0;
 	base::flat_map<ThreadId, MsgId> hiddenPinnedMessages;
+	base::flat_set<PeerId> verticalSubsectionTabs;
 	qint32 dialogsFiltersEnabled = _dialogsFiltersEnabled ? 1 : 0;
 	qint32 supportAllSilent = _supportAllSilent ? 1 : 0;
 	qint32 photoEditorHintShowsCount = _photoEditorHintShowsCount;
@@ -401,6 +408,7 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 		auto count = qint32(0);
 		stream >> count;
 		if (stream.status() == QDataStream::Ok) {
+			// Legacy.
 			for (auto i = 0; i != count; ++i) {
 				auto keyPeerId = quint64();
 				auto keyTopicRootId = qint64();
@@ -438,6 +446,49 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 			>> lastNonPremiumLimitDownload
 			>> lastNonPremiumLimitUpload;
 	}
+	if (!stream.atEnd()) {
+		auto count = qint32(0);
+		stream >> count;
+		if (stream.status() == QDataStream::Ok) {
+			for (auto i = 0; i != count; ++i) {
+				auto keyPeerId = quint64();
+				auto keyTopicRootId = qint64();
+				auto keyMonoforumPeerId = quint64();
+				auto value = qint64();
+				stream
+					>> keyPeerId
+					>> keyTopicRootId
+					>> keyMonoforumPeerId
+					>> value;
+				if (stream.status() != QDataStream::Ok) {
+					LOG(("App Error: "
+						"Bad data for SessionSettings::addFromSerialized()"));
+					return;
+				}
+				hiddenPinnedMessages.emplace(ThreadId{
+					DeserializePeerId(keyPeerId),
+					keyTopicRootId,
+					DeserializePeerId(keyMonoforumPeerId),
+				}, value);
+			}
+		}
+	}
+	if (!stream.atEnd()) {
+		auto count = qint32(0);
+		stream >> count;
+		if (stream.status() == QDataStream::Ok) {
+			for (auto i = 0; i != count; ++i) {
+				auto peerId = quint64();
+				stream >> peerId;
+				if (stream.status() != QDataStream::Ok) {
+					LOG(("App Error: "
+						"Bad data for SessionSettings::addFromSerialized()"));
+					return;
+				}
+				verticalSubsectionTabs.emplace(DeserializePeerId(peerId));
+			}
+		}
+	}
 	if (stream.status() != QDataStream::Ok) {
 		LOG(("App Error: "
 			"Bad data for SessionSettings::addFromSerialized()"));
@@ -484,6 +535,7 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 	_mutePeriods = std::move(mutePeriods);
 	_lastNonPremiumLimitDownload = lastNonPremiumLimitDownload;
 	_lastNonPremiumLimitUpload = lastNonPremiumLimitUpload;
+	_verticalSubsectionTabs = std::move(verticalSubsectionTabs);
 
 	if (version < 2) {
 		app.setLastSeenWarningSeen(appLastSeenWarningSeen == 1);
@@ -595,16 +647,22 @@ rpl::producer<bool> SessionSettings::skipArchiveInSearchChanges() const {
 
 MsgId SessionSettings::hiddenPinnedMessageId(
 		PeerId peerId,
-		MsgId topicRootId) const {
-	const auto i = _hiddenPinnedMessages.find({ peerId, topicRootId });
+		MsgId topicRootId,
+		PeerId monoforumPeerId) const {
+	const auto i = _hiddenPinnedMessages.find({
+		peerId,
+		topicRootId,
+		monoforumPeerId,
+	});
 	return (i != end(_hiddenPinnedMessages)) ? i->second : 0;
 }
 
 void SessionSettings::setHiddenPinnedMessageId(
 		PeerId peerId,
 		MsgId topicRootId,
+		PeerId monoforumPeerId,
 		MsgId msgId) {
-	const auto id = ThreadId{ peerId, topicRootId };
+	const auto id = ThreadId{ peerId, topicRootId, monoforumPeerId };
 	if (msgId) {
 		_hiddenPinnedMessages[id] = msgId;
 	} else {
@@ -612,6 +670,20 @@ void SessionSettings::setHiddenPinnedMessageId(
 	}
 }
 
+bool SessionSettings::verticalSubsectionTabs(PeerId peerId) const {
+	return _verticalSubsectionTabs.contains(peerId);
+}
+
+void SessionSettings::setVerticalSubsectionTabs(
+		PeerId peerId,
+		bool vertical) {
+	if (vertical) {
+		_verticalSubsectionTabs.emplace(peerId);
+	} else {
+		_verticalSubsectionTabs.remove(peerId);
+	}
+}
+
 bool SessionSettings::photoEditorHintShown() const {
 	return _photoEditorHintShowsCount < kPhotoEditorHintMaxShowsCount;
 }
