@@ -30,8 +30,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/history_item_components.h"
 #include "history/history_item_helpers.h"
 #include "history/view/history_view_cursor_state.h"
+#include "history/view/media/history_view_media_common.h"
 #include "history/view/media/history_view_document.h" // DrawThumbnailAsSongCover
 #include "base/unixtime.h"
+#include "boxes/sticker_set_box.h"
 #include "ui/effects/round_checkbox.h"
 #include "ui/effects/spoiler_mess.h"
 #include "ui/image/image.h"
@@ -98,6 +100,27 @@ constexpr auto kStoryRatio = 1.46;
 	}
 }
 
+void PaintSensitiveTag(Painter &p, QRect r) {
+	auto text = Ui::Text::String();
+	text.setText(
+		st::semiboldTextStyle,
+		tr::lng_sensitive_tag(tr::now));
+	const auto width = text.maxWidth();
+	const auto inner = QRect(0, 0, width, text.minHeight());
+	const auto outer = style::centerrect(r, inner.marginsAdded(st::paidTagPadding));
+	const auto size = outer.size();
+	const auto radius = std::min(size.width(), size.height()) / 2;
+	auto hq = PainterHighQualityEnabler(p);
+
+	p.setPen(Qt::NoPen);
+	p.setBrush(st::radialBg);
+	p.drawRoundedRect(outer, radius, radius);
+	p.setPen(st::radialFg);
+	text.draw(p, {
+		.position = outer.marginsRemoved(st::paidTagPadding).topLeft(),
+	});
+}
+
 } // namespace
 
 class Checkbox {
@@ -327,18 +350,24 @@ Photo::Photo(
 	MediaOptions options)
 : ItemBase(delegate, parent)
 , _data(photo)
-, _link(std::make_shared<PhotoOpenClickHandler>(
-	photo,
-	crl::guard(this, [=](FullMsgId id) {
-		clearSpoiler();
-		delegate->openPhoto(photo, id);
-	}),
-	parent->fullId()))
-, _spoiler(options.spoiler ? std::make_unique<Ui::SpoilerAnimation>([=] {
-	delegate->repaintItem(this);
-}) : nullptr)
-, _pinned(options.pinned)
-, _story(options.story) {
+, _spoiler((options.spoiler || parent->isMediaSensitive())
+	? std::make_unique<Ui::SpoilerAnimation>([=] {
+		delegate->repaintItem(this);
+	})
+	: nullptr)
+, _sensitiveSpoiler(parent->isMediaSensitive() ? 1 : 0)
+, _story(options.story)
+, _storyPinned(options.storyPinned)
+, _storyShowPinned(options.storyShowPinned)
+, _storyHidden(options.storyHidden)
+, _storyShowHidden(options.storyShowHidden)
+, _link(_sensitiveSpoiler
+	? HistoryView::MakeSensitiveMediaLink(
+		std::make_shared<LambdaClickHandler>(crl::guard(this, [=] {
+			maybeClearSensitiveSpoiler();
+		})),
+		parent)
+	: makeOpenPhotoHandler()) {
 	if (_data->inlineThumbnailBytes().isEmpty()
 		&& (_data->hasExact(Data::PhotoSize::Small)
 			|| _data->hasExact(Data::PhotoSize::Thumbnail))) {
@@ -348,6 +377,16 @@ Photo::Photo(
 
 Photo::~Photo() = default;
 
+ClickHandlerPtr Photo::makeOpenPhotoHandler() {
+	return std::make_shared<PhotoOpenClickHandler>(
+		_data,
+		crl::guard(this, [=](FullMsgId id) {
+			clearSpoiler();
+			delegate()->openPhoto(_data, id);
+		}),
+		parent()->fullId());
+}
+
 void Photo::initDimensions() {
 	_maxw = 2 * st::overviewPhotoMinSize;
 	_minh = _story ? qRound(_maxw * kStoryRatio) : _maxw;
@@ -373,7 +412,7 @@ void Photo::paint(Painter &p, const QRect &clip, TextSelection selection, const
 				|| _dataMedia->image(Data::PhotoSize::Thumbnail));
 		if ((good && !_goodLoaded) || widthChanged) {
 			_goodLoaded = good;
-			_pix = QPixmap();
+			_pix = QImage();
 			if (_goodLoaded) {
 				setPixFrom(_dataMedia->image(Data::PhotoSize::Large)
 					? _dataMedia->image(Data::PhotoSize::Large)
@@ -391,7 +430,7 @@ void Photo::paint(Painter &p, const QRect &clip, TextSelection selection, const
 	if (_pix.isNull()) {
 		p.fillRect(0, 0, _width, _height, st::overviewPhotoBg);
 	} else {
-		p.drawPixmap(0, 0, _pix);
+		p.drawImage(0, 0, _pix);
 	}
 
 	if (_spoiler) {
@@ -401,13 +440,27 @@ void Photo::paint(Painter &p, const QRect &clip, TextSelection selection, const
 			QRect(0, 0, _width, _height),
 			Ui::DefaultImageSpoiler().frame(
 				_spoiler->index(context->ms, paused)));
+
+		if (_sensitiveSpoiler) {
+			PaintSensitiveTag(p, QRect(0, 0, _width, _height));
+		}
+	}
+
+	if (_storyHidden) {
+		delegate()->hiddenMark()->paint(
+			p,
+			_pix,
+			_hiddenBgCache,
+			QPoint(),
+			QSize(_width, _height),
+			_width);
 	}
 
 	if (selected) {
 		p.fillRect(0, 0, _width, _height, st::overviewPhotoSelectOverlay);
 	}
 
-	if (_pinned) {
+	if (_storyPinned) {
 		const auto &icon = selected
 			? st::storyPinnedIconSelected
 			: st::storyPinnedIcon;
@@ -427,8 +480,7 @@ void Photo::setPixFrom(not_null<Image*> image) {
 	if (!_goodLoaded) {
 		img = Images::Blur(std::move(img));
 	}
-	_pix = Ui::PixmapFromImage(
-		CropMediaFrame(std::move(img), _width, _height));
+	_pix = CropMediaFrame(std::move(img), _width, _height);
 
 	// In case we have inline thumbnail we can unload all images and we still
 	// won't get a blank image in the media viewer when the photo is opened.
@@ -453,15 +505,25 @@ void Photo::ensureDataMediaCreated() const {
 void Photo::clearSpoiler() {
 	if (_spoiler) {
 		_spoiler = nullptr;
-		_pix = QPixmap();
+		_sensitiveSpoiler = false;
+		_pix = QImage();
 		delegate()->repaintItem(this);
 	}
 }
 
+void Photo::maybeClearSensitiveSpoiler() {
+	if (_sensitiveSpoiler) {
+		clearSpoiler();
+		_link = makeOpenPhotoHandler();
+	}
+}
+
 void Photo::itemDataChanged() {
-	const auto pinned = parent()->isPinned();
-	if (_pinned != pinned) {
-		_pinned = pinned;
+	const auto pinned = _storyShowPinned && parent()->isPinned();
+	const auto hidden = _storyShowHidden && !parent()->storyInProfile();
+	if (_storyPinned != pinned || _storyHidden != hidden) {
+		_storyPinned = pinned;
+		_storyHidden = hidden;
 		delegate()->repaintItem(this);
 	}
 }
@@ -488,12 +550,26 @@ Video::Video(
 , _data(video)
 , _videoCover(LookupVideoCover(video, parent))
 , _duration(Ui::FormatDurationText(_data->duration() / 1000))
-, _spoiler(options.spoiler ? std::make_unique<Ui::SpoilerAnimation>([=] {
-	delegate->repaintItem(this);
-}) : nullptr)
-, _pinned(options.pinned)
-, _story(options.story) {
+, _spoiler((options.spoiler || parent->isMediaSensitive())
+	? std::make_unique<Ui::SpoilerAnimation>([=] {
+		delegate->repaintItem(this);
+	})
+	: nullptr)
+, _sensitiveSpoiler(parent->isMediaSensitive() ? 1 : 0)
+, _story(options.story)
+, _storyPinned(options.storyPinned)
+, _storyShowPinned(options.storyShowPinned)
+, _storyHidden(options.storyHidden)
+, _storyShowHidden(options.storyShowHidden) {
 	setDocumentLinks(_data);
+	if (_sensitiveSpoiler) {
+		_openl = HistoryView::MakeSensitiveMediaLink(
+			std::make_shared<LambdaClickHandler>(crl::guard(this, [=] {
+				clearSpoiler();
+				setDocumentLinks(_data);
+			})),
+			parent);
+	}
 	if (!_videoCover) {
 		_data->loadThumbnail(parent->fullId());
 	} else if (_videoCover->inlineThumbnailBytes().isEmpty()
@@ -519,7 +595,11 @@ int32 Video::resizeGetHeight(int32 width) {
 	return _height;
 }
 
-void Video::paint(Painter &p, const QRect &clip, TextSelection selection, const PaintContext *context) {
+void Video::paint(
+		Painter &p,
+		const QRect &clip,
+		TextSelection selection,
+		const PaintContext *context) {
 	ensureDataMediaCreated();
 
 	const auto selected = (selection == FullSelection);
@@ -556,15 +636,14 @@ void Video::paint(Painter &p, const QRect &clip, TextSelection selection, const
 			: thumbnail
 			? thumbnail->original()
 			: Images::Blur(blurred->original());
-		_pix = Ui::PixmapFromImage(
-			CropMediaFrame(std::move(img), _width, _height));
+		_pix = CropMediaFrame(std::move(img), _width, _height);
 		_pixBlurred = !(thumbnail || good);
 	}
 
 	if (_pix.isNull()) {
 		p.fillRect(0, 0, _width, _height, st::overviewPhotoBg);
 	} else {
-		p.drawPixmap(0, 0, _pix);
+		p.drawImage(0, 0, _pix);
 	}
 
 	if (_spoiler) {
@@ -574,13 +653,27 @@ void Video::paint(Painter &p, const QRect &clip, TextSelection selection, const
 			QRect(0, 0, _width, _height),
 			Ui::DefaultImageSpoiler().frame(
 				_spoiler->index(context->ms, paused)));
+
+		if (_sensitiveSpoiler) {
+			PaintSensitiveTag(p, QRect(0, 0, _width, _height));
+		}
+	}
+
+	if (_storyHidden) {
+		delegate()->hiddenMark()->paint(
+			p,
+			_pix,
+			_hiddenBgCache,
+			QPoint(),
+			QSize(_width, _height),
+			_width);
 	}
 
 	if (selected) {
 		p.fillRect(QRect(0, 0, _width, _height), st::overviewPhotoSelectOverlay);
 	}
 
-	if (_pinned) {
+	if (_storyPinned) {
 		const auto &icon = selected
 			? st::storyPinnedIconSelected
 			: st::storyPinnedIcon;
@@ -660,15 +753,25 @@ void Video::ensureDataMediaCreated() const {
 void Video::clearSpoiler() {
 	if (_spoiler) {
 		_spoiler = nullptr;
-		_pix = QPixmap();
+		_sensitiveSpoiler = false;
+		_pix = QImage();
 		delegate()->repaintItem(this);
 	}
 }
 
+void Video::maybeClearSensitiveSpoiler() {
+	if (_sensitiveSpoiler) {
+		clearSpoiler();
+		setDocumentLinks(_data);
+	}
+}
+
 void Video::itemDataChanged() {
-	const auto pinned = parent()->isPinned();
-	if (_pinned != pinned) {
-		_pinned = pinned;
+	const auto pinned = _storyShowPinned && parent()->isPinned();
+	const auto hidden = _storyShowHidden && !parent()->storyInProfile();
+	if (_storyPinned != pinned || _storyHidden != hidden) {
+		_storyPinned = pinned;
+		_storyHidden = hidden;
 		delegate()->repaintItem(this);
 	}
 }
@@ -700,7 +803,9 @@ TextState Video::getState(
 		StateRequest request) const {
 	if (hasPoint(point)) {
 		ensureDataMediaCreated();
-		const auto link = (_data->loading() || _data->uploading())
+		const auto link = _sensitiveSpoiler
+			? _openl
+			: (_data->loading() || _data->uploading())
 			? _cancell
 			: (dataLoaded() || _dataMedia->canBePlayed(parent()))
 			? _openl
@@ -2036,8 +2141,22 @@ Gif::Gif(
 	not_null<HistoryItem*> parent,
 	not_null<DocumentData*> gif)
 : RadialProgressItem(delegate, parent)
-, _data(gif) {
+, _data(gif)
+, _spoiler(parent->isMediaSensitive()
+	? std::make_unique<Ui::SpoilerAnimation>([=] {
+		delegate->repaintItem(this);
+	})
+	: nullptr)
+, _sensitiveSpoiler(parent->isMediaSensitive() ? 1 : 0) {
 	setDocumentLinks(_data, true);
+	if (_sensitiveSpoiler) {
+		_openl = HistoryView::MakeSensitiveMediaLink(
+			std::make_shared<LambdaClickHandler>(crl::guard(this, [=] {
+				clearSpoiler();
+				setDocumentLinks(_data, true);
+			})),
+			parent);
+	}
 	_data->loadThumbnail(parent->fullId());
 }
 
@@ -2140,6 +2259,23 @@ void Gif::clipCallback(Media::Clip::Notification notification) {
 	}
 }
 
+void Gif::clearSpoiler() {
+	if (_spoiler) {
+		_spoiler = nullptr;
+		_sensitiveSpoiler = false;
+		_thumb = QImage();
+		_thumbGood = false;
+		delegate()->repaintItem(this);
+	}
+}
+
+void Gif::maybeClearSensitiveSpoiler() {
+	if (_sensitiveSpoiler) {
+		clearSpoiler();
+		setDocumentLinks(_data);
+	}
+}
+
 void Gif::validateThumbnail(
 		Image *image,
 		QSize size,
@@ -2165,7 +2301,9 @@ void Gif::prepareThumbnail(QSize size, QSize frame) {
 	Assert(document != nullptr);
 
 	ensureDataMediaCreated();
-	validateThumbnail(_dataMedia->thumbnail(), size, frame, true);
+	if (!_spoiler) {
+		validateThumbnail(_dataMedia->thumbnail(), size, frame, true);
+	}
 	validateThumbnail(_dataMedia->thumbnailInline(), size, frame, false);
 }
 
@@ -2194,7 +2332,7 @@ void Gif::paint(
 		});
 	}
 
-	const auto animating = (_gif && _gif->started());
+	const auto animating = !_spoiler && (_gif && _gif->started());
 	if (displayLoading) {
 		ensureRadial();
 		if (!_radial->animating()) {
@@ -2224,6 +2362,19 @@ void Gif::paint(
 		}
 	}
 
+	if (_spoiler) {
+		const auto paused = context->paused || On(PowerSaving::kChatSpoiler);
+		Ui::FillSpoilerRect(
+			p,
+			r,
+			Ui::DefaultImageSpoiler().frame(
+				_spoiler->index(context->ms, paused)));
+
+		if (_sensitiveSpoiler) {
+			PaintSensitiveTag(p, r);
+		}
+	}
+
 	const auto selected = (selection == FullSelection);
 
 	if (radial
