@@ -76,6 +76,9 @@ public:
 	virtual void clearHeavyPart() {
 	}
 
+	virtual void maybeClearSensitiveSpoiler() {
+	}
+
 protected:
 	[[nodiscard]] not_null<HistoryItem*> parent() const {
 		return _parent;
@@ -187,8 +190,11 @@ struct Info : RuntimeComponent<Info, LayoutItemBase> {
 
 struct MediaOptions {
 	bool spoiler = false;
-	bool pinned = false;
 	bool story = false;
+	bool storyPinned = false;
+	bool storyShowPinned = false;
+	bool storyHidden = false;
+	bool storyShowHidden = false;
 };
 
 class Photo final : public ItemBase {
@@ -210,20 +216,29 @@ public:
 	void itemDataChanged() override;
 	void clearHeavyPart() override;
 
+	void maybeClearSensitiveSpoiler() override;
+
 private:
 	void ensureDataMediaCreated() const;
 	void setPixFrom(not_null<Image*> image);
+	[[nodiscard]] ClickHandlerPtr makeOpenPhotoHandler();
 	void clearSpoiler();
 
 	const not_null<PhotoData*> _data;
 	mutable std::shared_ptr<Data::PhotoMedia> _dataMedia;
-	ClickHandlerPtr _link;
 	std::unique_ptr<Ui::SpoilerAnimation> _spoiler;
 
-	QPixmap _pix;
-	bool _goodLoaded = false;
-	bool _pinned = false;
-	bool _story = false;
+	QImage _pix;
+	QImage _hiddenBgCache;
+	bool _goodLoaded : 1 = false;
+	bool _sensitiveSpoiler : 1 = false;
+	bool _story : 1 = false;
+	bool _storyPinned : 1 = false;
+	bool _storyShowPinned : 1 = false;
+	bool _storyHidden : 1 = false;
+	bool _storyShowHidden : 1 = false;
+
+	ClickHandlerPtr _link;
 
 };
 
@@ -249,6 +264,9 @@ public:
 	void clearHeavyPart() override;
 	void setPosition(int32 position) override;
 
+	void clearSpoiler() override;
+	void maybeClearSensitiveSpoiler() override;
+
 protected:
 	float64 dataProgress() const override;
 	bool dataFinished() const override;
@@ -279,9 +297,11 @@ private:
 	const not_null<DocumentData*> _data;
 	mutable std::shared_ptr<Data::DocumentMedia> _dataMedia;
 	StatusText _status;
+	std::unique_ptr<Ui::SpoilerAnimation> _spoiler;
 
 	QImage _thumb;
 	bool _thumbGood = false;
+	bool _sensitiveSpoiler = false;
 
 };
 
@@ -305,6 +325,8 @@ public:
 	void clearHeavyPart() override;
 	void clearSpoiler() override;
 
+	void maybeClearSensitiveSpoiler() override;
+
 protected:
 	float64 dataProgress() const override;
 	bool dataFinished() const override;
@@ -324,10 +346,15 @@ private:
 	QString _duration;
 	std::unique_ptr<Ui::SpoilerAnimation> _spoiler;
 
-	QPixmap _pix;
-	bool _pixBlurred = true;
-	bool _pinned = false;
-	bool _story = false;
+	QImage _pix;
+	QImage _hiddenBgCache;
+	bool _pixBlurred : 1 = true;
+	bool _sensitiveSpoiler : 1 = false;
+	bool _story : 1 = false;
+	bool _storyPinned : 1 = false;
+	bool _storyShowPinned : 1 = false;
+	bool _storyHidden : 1 = false;
+	bool _storyShowHidden : 1 = false;
 
 };
 
