@@ -11,10 +11,13 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "base/options.h"
 #include "base/platform/base_platform_info.h"
 #include "base/platform/linux/base_linux_dbus_utilities.h"
+#include "platform/platform_specific.h"
 #include "core/application.h"
 #include "core/sandbox.h"
 #include "core/core_settings.h"
 #include "data/data_forum_topic.h"
+#include "data/data_saved_sublist.h"
+#include "data/data_peer.h"
 #include "history/history.h"
 #include "history/history_item.h"
 #include "main/main_session.h"
@@ -27,6 +30,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include <QtCore/QVersionNumber>
 #include <QtGui/QGuiApplication>
 
+#include <ksandbox.h>
+
 #include <xdgnotifications/xdgnotifications.hpp>
 
 #include <dlfcn.h>
@@ -153,6 +158,7 @@ public:
 	void clearAll();
 	void clearFromItem(not_null<HistoryItem*> item);
 	void clearFromTopic(not_null<Data::ForumTopic*> topic);
+	void clearFromSublist(not_null<Data::SavedSublist*> sublist);
 	void clearFromHistory(not_null<History*> history);
 	void clearFromSession(not_null<Main::Session*> session);
 	void clearNotification(NotificationId id);
@@ -363,7 +369,10 @@ Manager::Private::Private(not_null<Manager*> manager)
 				.contextId = ContextId{
 					.sessionId = dict.lookup_value("session").get_uint64(),
 					.peerId = PeerId(dict.lookup_value("peer").get_uint64()),
-					.topicRootId = dict.lookup_value("topic").get_int64(),
+					.topicRootId = MsgId(
+						dict.lookup_value("topic").get_int64()),
+					.monoforumPeerId = PeerId(dict.lookup_value(
+						"monoforumpeer").get_uint64()),
 				},
 				.msgId = dict.lookup_value("msgid").get_int64(),
 			};
@@ -528,26 +537,33 @@ void Manager::Private::showNotification(
 		.sessionId = peer->session().uniqueId(),
 		.peerId = peer->id,
 		.topicRootId = info.topicRootId,
+		.monoforumPeerId = info.monoforumPeerId,
 	};
 	const auto notificationId = NotificationId{
 		.contextId = key,
 		.msgId = info.itemId,
 	};
 	auto notification = _application
-		? Gio::Notification::new_(
-			info.subtitle.isEmpty()
-				? info.title.toStdString()
-				: info.subtitle.toStdString()
-					+ " (" + info.title.toStdString() + ')')
+		? Gio::Notification::new_(info.title.toStdString())
 		: Gio::Notification();
 
 	std::vector<gi::cstring> actions;
 	auto hints = GLib::VariantDict::new_();
 	if (notification) {
-		notification.set_body(info.message.toStdString());
+		notification.set_body(info.subtitle.isEmpty()
+			? info.message.toStdString()
+			: tr::lng_dialogs_text_with_from(
+				tr::now,
+				lt_from_part,
+				tr::lng_dialogs_text_from_wrapped(
+					tr::now,
+					lt_from,
+					info.subtitle),
+				lt_message,
+				info.message).toStdString());
 
 		notification.set_icon(
-			Gio::ThemedIcon::new_(base::IconName().toStdString()));
+			Gio::ThemedIcon::new_(ApplicationIconName().toStdString()));
 
 		// for chat messages, according to
 		// https://docs.gtk.org/gio/enum.NotificationPriority.html
@@ -582,6 +598,10 @@ void Manager::Private::showNotification(
 				GLib::Variant::new_string("topic"),
 				GLib::Variant::new_variant(
 					GLib::Variant::new_int64(info.topicRootId.bare))),
+			GLib::Variant::new_dict_entry(
+				GLib::Variant::new_string("monoforumpeer"),
+				GLib::Variant::new_variant(
+					GLib::Variant::new_uint64(info.monoforumPeerId.value))),
 			GLib::Variant::new_dict_entry(
 				GLib::Variant::new_string("msgid"),
 				GLib::Variant::new_variant(
@@ -616,10 +636,10 @@ void Manager::Private::showNotification(
 				actions.push_back(
 					tr::lng_notification_reply(tr::now).toStdString());
 			}
-
-			actions.push_back({});
 		}
 
+		actions.push_back({});
+
 		if (HasCapability("action-icons")) {
 			hints.insert_value(
 				"action-icons",
@@ -723,8 +743,6 @@ void Manager::Private::showNotification(
 				const auto hasImage = !imageKey.empty()
 					&& hints.lookup_value(imageKey);
 
-				const auto hasBodyMarkup = HasCapability("body-markup");
-
 				const auto callbackWrap = gi::unwrap(
 					Gio::AsyncReadyCallback(
 						crl::guard(this, [=](
@@ -761,24 +779,29 @@ void Manager::Private::showNotification(
 					AppName.data(),
 					0,
 					(!hasImage
-						? base::IconName().toStdString()
+						? ApplicationIconName().toStdString()
 						: std::string()).c_str(),
-					(hasBodyMarkup || info.subtitle.isEmpty()
-						? info.title.toStdString()
-						: info.subtitle.toStdString()
-							+ " (" + info.title.toStdString() + ')').c_str(),
-					(hasBodyMarkup
+					info.title.toStdString().c_str(),
+					(HasCapability("body-markup")
 						? info.subtitle.isEmpty()
 							? info.message.toHtmlEscaped().toStdString()
 							: u"<b>%1</b>\n%2"_q.arg(
 								info.subtitle.toHtmlEscaped(),
 								info.message.toHtmlEscaped()).toStdString()
-						: info.message.toStdString()).c_str(),
-					!actions.empty()
-						? (actions
-							| ranges::views::transform(&gi::cstring::c_str)
-							| ranges::to_vector).data()
-						: nullptr,
+						: info.subtitle.isEmpty()
+							? info.message.toStdString()
+							: tr::lng_dialogs_text_with_from(
+								tr::now,
+								lt_from_part,
+								tr::lng_dialogs_text_from_wrapped(
+									tr::now,
+									lt_from,
+									info.subtitle),
+								lt_message,
+								info.message).toStdString()).c_str(),
+					(actions
+						| ranges::views::transform(&gi::cstring::c_str)
+						| ranges::to_vector).data(),
 					hints.end().gobj_(),
 					-1,
 					nullptr,
@@ -797,6 +820,7 @@ void Manager::Private::clearFromItem(not_null<HistoryItem*> item) {
 		.sessionId = item->history()->session().uniqueId(),
 		.peerId = item->history()->peer->id,
 		.topicRootId = item->topicRootId(),
+		.monoforumPeerId = item->sublistPeerId(),
 	});
 	if (i != _notifications.cend()
 			&& i->second.remove(item->id)
@@ -813,6 +837,15 @@ void Manager::Private::clearFromTopic(not_null<Data::ForumTopic*> topic) {
 	});
 }
 
+void Manager::Private::clearFromSublist(
+		not_null<Data::SavedSublist*> sublist) {
+	_notifications.remove(ContextId{
+		.sessionId = sublist->session().uniqueId(),
+		.peerId = sublist->owningHistory()->peer->id,
+		.monoforumPeerId = sublist->sublistPeer()->id,
+	});
+}
+
 void Manager::Private::clearFromHistory(not_null<History*> history) {
 	const auto sessionId = history->session().uniqueId();
 	const auto peerId = history->peer->id;
@@ -877,6 +910,10 @@ void Manager::doClearFromTopic(not_null<Data::ForumTopic*> topic) {
 	_private->clearFromTopic(topic);
 }
 
+void Manager::doClearFromSublist(not_null<Data::SavedSublist*> sublist) {
+	_private->clearFromSublist(sublist);
+}
+
 void Manager::doClearFromHistory(not_null<History*> history) {
 	_private->clearFromHistory(history);
 }
