@@ -91,6 +91,8 @@ using SectionCustomTopBarData = Info::Settings::SectionCustomTopBarData;
 			option.duration = tr::lng_premium_subscribe_months_6(tr::now);
 		} else if (option.duration == tr::lng_years(tr::now, lt_count, 1)) {
 			option.duration = tr::lng_premium_subscribe_months_12(tr::now);
+		} else if (option.duration == tr::lng_years(tr::now, lt_count, 2)) {
+			option.duration = tr::lng_premium_subscribe_months_24(tr::now);
 		}
 	}
 	return result;
@@ -164,6 +166,45 @@ struct Data {
 }
 
 } // namespace EmojiStatus
+
+namespace PremiumGift {
+
+struct Data {
+	DocumentId documentId = 0;
+	int perUserTotal = 0;
+
+	explicit operator bool() const {
+		return documentId != 0;
+	}
+};
+
+[[nodiscard]] QString Serialize(const Data &gift) {
+	return u"premiumgift_:%1,peruser_:%2"_q
+		.arg(gift.documentId)
+		.arg(gift.perUserTotal);
+}
+
+[[nodiscard]] Data Parse(QStringView data) {
+	if (data.startsWith(u"premiumgift_:"_q)) {
+		const auto components = data.split(',');
+		if (components.size() != 2) {
+			return {};
+		}
+		const auto first = components[0].split(':');
+		const auto second = components[1].split(':');
+		if (first.size() != 2 || second.size() != 2) {
+			return {};
+		}
+		return {
+			.documentId = DocumentId(first[1].toULongLong()),
+			.perUserTotal = second[1].toInt(),
+		};
+	}
+	return {};
+}
+
+} // namespace PremiumGift
+
 } // namespace Ref
 
 struct Entry {
@@ -374,7 +415,6 @@ using Order = std::vector<QString>;
 				tr::lng_premium_summary_subtitle_business(),
 				tr::lng_premium_summary_about_business(),
 				PremiumFeature::Business,
-				true,
 			},
 		},
 		{
@@ -393,6 +433,7 @@ using Order = std::vector<QString>;
 				tr::lng_premium_summary_subtitle_todo_lists(),
 				tr::lng_premium_summary_about_todo_lists(),
 				PremiumFeature::TodoLists,
+				true,
 			},
 		},
 	};
@@ -542,13 +583,30 @@ void EmojiStatusTopBar::paint(QPainter &p) {
 	}
 }
 
-class TopBarUser final : public Ui::Premium::TopBarAbstract {
+enum class TopBarWithStickerType {
+	EmojiStatus,
+	PremiumGift,
+};
+
+struct TopBarWithStickerArgs {
+	rpl::producer<DocumentData*> stickerValue;
+	rpl::producer<QString> nameValue;
+	rpl::producer<TextWithEntities> aboutValue;
+	TopBarWithStickerType type = TopBarWithStickerType::EmojiStatus;
+};
+
+class TopBarWithSticker final : public Ui::Premium::TopBarAbstract {
 public:
-	TopBarUser(
+	TopBarWithSticker(
 		not_null<QWidget*> parent,
 		not_null<Window::SessionController*> controller,
 		not_null<PeerData*> peer,
 		rpl::producer<> showFinished);
+	TopBarWithSticker(
+		not_null<QWidget*> parent,
+		not_null<Window::SessionController*> controller,
+		TopBarWithStickerArgs args,
+		rpl::producer<> showFinished);
 
 	void setPaused(bool paused) override;
 	void setTextPosition(int x, int y) override;
@@ -562,10 +620,13 @@ protected:
 private:
 	void updateTitle(
 		DocumentData *document,
-		TextWithEntities name,
+		const TextWithEntities &name,
 		not_null<Window::SessionController*> controller);
-	void updateAbout(DocumentData *document) const;
+	void updateAbout(
+		DocumentData *document,
+		const TextWithEntities &about) const;
 
+	TopBarWithStickerType _type = TopBarWithStickerType::EmojiStatus;
 	object_ptr<Ui::RpWidget> _content;
 	object_ptr<Ui::FlatLabel> _title;
 	object_ptr<Ui::FlatLabel> _about;
@@ -586,12 +647,35 @@ private:
 
 };
 
-TopBarUser::TopBarUser(
+TopBarWithSticker::TopBarWithSticker(
 	not_null<QWidget*> parent,
 	not_null<Window::SessionController*> controller,
 	not_null<PeerData*> peer,
 	rpl::producer<> showFinished)
+: TopBarWithSticker(parent, controller, {
+	.stickerValue = Info::Profile::EmojiStatusIdValue(
+		peer
+	) | rpl::map([=](EmojiStatusId id) -> DocumentData* {
+		const auto documentId = id.collectible
+			? id.collectible->documentId
+			: id.documentId;
+		const auto document = documentId
+			? controller->session().data().document(documentId).get()
+			: nullptr;
+		return (document && document->sticker()) ? document : nullptr;
+	}),
+	.nameValue = Info::Profile::NameValue(peer),
+	.type = TopBarWithStickerType::EmojiStatus,
+}, std::move(showFinished)) {
+}
+
+TopBarWithSticker::	TopBarWithSticker(
+	not_null<QWidget*> parent,
+	not_null<Window::SessionController*> controller,
+	TopBarWithStickerArgs args,
+	rpl::producer<> showFinished)
 : TopBarAbstract(parent, st::userPremiumCover)
+, _type(args.type)
 , _content(this)
 , _title(_content, st::settingsPremiumUserTitle)
 , _about(_content, st::userPremiumCover.about)
@@ -604,6 +688,10 @@ TopBarUser::TopBarUser(
 }) {
 	_starRect = TopBarAbstract::starRect(1., 1.);
 
+	if (_type == TopBarWithStickerType::PremiumGift) {
+		_ministars.setColorOverride(Ui::Premium::CreditsIconGradientStops());
+	}
+
 	rpl::single() | rpl::then(
 		style::PaletteChanged()
 	) | rpl::start_with_next([=] {
@@ -611,24 +699,20 @@ TopBarUser::TopBarUser(
 		update();
 	}, lifetime());
 
-	auto documentValue = Info::Profile::EmojiStatusIdValue(
-		peer
-	) | rpl::map([=](EmojiStatusId id) -> DocumentData* {
-		const auto documentId = id.collectible
-			? id.collectible->documentId
-			: id.documentId;
-		const auto document = documentId
-			? controller->session().data().document(documentId).get()
-			: nullptr;
-		return (document && document->sticker()) ? document : nullptr;
-	});
-
 	rpl::combine(
-		std::move(documentValue),
-		Info::Profile::NameValue(peer)
+		(args.stickerValue
+			? std::move(args.stickerValue)
+			: rpl::single((DocumentData*)nullptr)),
+		(args.nameValue
+			? std::move(args.nameValue)
+			: rpl::single(QString())),
+		(args.aboutValue
+			? std::move(args.aboutValue)
+			: rpl::single(TextWithEntities()))
 	) | rpl::start_with_next([=](
 			DocumentData *document,
-			const QString &name) {
+			const QString &name,
+			const TextWithEntities &about) {
 		if (document) {
 			_emojiStatus = std::make_unique<EmojiStatusTopBar>(
 				document,
@@ -641,7 +725,7 @@ TopBarUser::TopBarUser(
 		}
 
 		updateTitle(document, { name }, controller);
-		updateAbout(document);
+		updateAbout(document, about);
 
 		auto event = QResizeEvent(size(), size());
 		resizeEvent(&event);
@@ -746,14 +830,15 @@ TopBarUser::TopBarUser(
 			p.drawImage(_starRect.topLeft(), _imageStar);
 		}
 	}, lifetime());
-
 }
 
-void TopBarUser::updateTitle(
+void TopBarWithSticker::updateTitle(
 		DocumentData *document,
-		TextWithEntities name,
+		const TextWithEntities &name,
 		not_null<Window::SessionController*> controller) {
-	if (!document) {
+	if (_type == TopBarWithStickerType::PremiumGift) {
+		return _title->setMarkedText(name);
+	} else if (!document) {
 		return _title->setMarkedText(
 			tr::lng_premium_summary_user_title(
 				tr::now,
@@ -816,36 +901,40 @@ void TopBarUser::updateTitle(
 	_title->setLink(linkIndex, std::move(link));
 }
 
-void TopBarUser::updateAbout(DocumentData *document) const {
-	_about->setMarkedText((document
-		? tr::lng_premium_emoji_status_about
-		: tr::lng_premium_summary_user_about)(
-			tr::now,
-			Ui::Text::RichLangValue));
+void TopBarWithSticker::updateAbout(
+		DocumentData *document,
+		const TextWithEntities &about) const {
+	_about->setMarkedText((_type == TopBarWithStickerType::PremiumGift)
+		? about
+		: (document
+			? tr::lng_premium_emoji_status_about
+			: tr::lng_premium_summary_user_about)(
+				tr::now,
+				Ui::Text::RichLangValue));
 }
 
-void TopBarUser::setPaused(bool paused) {
+void TopBarWithSticker::setPaused(bool paused) {
 	_ministars.setPaused(paused);
 	if (_emojiStatus) {
 		_emojiStatus->setPaused(paused);
 	}
 }
 
-void TopBarUser::setTextPosition(int x, int y) {
+void TopBarWithSticker::setTextPosition(int x, int y) {
 	_smallTop.position = { x, y };
 }
 
-rpl::producer<int> TopBarUser::additionalHeight() const {
+rpl::producer<int> TopBarWithSticker::additionalHeight() const {
 	return rpl::never<int>();
 }
 
-void TopBarUser::paintEvent(QPaintEvent *e) {
+void TopBarWithSticker::paintEvent(QPaintEvent *e) {
 	auto p = QPainter(this);
 
 	TopBarAbstract::paintEdges(p);
 }
 
-void TopBarUser::resizeEvent(QResizeEvent *e) {
+void TopBarWithSticker::resizeEvent(QResizeEvent *e) {
 	_starRect = TopBarAbstract::starRect(1., 1.);
 
 	_ministars.setCenter(_starRect.toRect());
@@ -863,9 +952,9 @@ public:
 
 	[[nodiscard]] rpl::producer<QString> title() override;
 
-	[[nodiscard]] QPointer<Ui::RpWidget> createPinnedToTop(
+	[[nodiscard]] base::weak_qptr<Ui::RpWidget> createPinnedToTop(
 		not_null<QWidget*> parent) override;
-	[[nodiscard]] QPointer<Ui::RpWidget> createPinnedToBottom(
+	[[nodiscard]] base::weak_qptr<Ui::RpWidget> createPinnedToBottom(
 		not_null<Ui::RpWidget*> parent) override;
 
 	void showFinished() override;
@@ -1023,7 +1112,7 @@ void Premium::setupContent() {
 
 }
 
-QPointer<Ui::RpWidget> Premium::createPinnedToTop(
+base::weak_qptr<Ui::RpWidget> Premium::createPinnedToTop(
 		not_null<QWidget*> parent) {
 	auto title = _controller->session().premium()
 		? tr::lng_premium_summary_title()
@@ -1053,7 +1142,9 @@ QPointer<Ui::RpWidget> Premium::createPinnedToTop(
 	}();
 
 	const auto emojiStatusData = Ref::EmojiStatus::Parse(_ref);
+	const auto premiumGiftData = Ref::PremiumGift::Parse(_ref);
 	const auto isEmojiStatus = (!!emojiStatusData);
+	const auto isPremiumGift = (!!premiumGiftData);
 
 	auto peerWithPremium = [&]() -> PeerData* {
 		if (isEmojiStatus) {
@@ -1064,14 +1155,35 @@ QPointer<Ui::RpWidget> Premium::createPinnedToTop(
 		}
 		return nullptr;
 	}();
+	auto premiumGift = [&]() -> DocumentData* {
+		if (isPremiumGift) {
+			auto &data = _controller->session().data();
+			return data.document(premiumGiftData.documentId);
+		}
+		return nullptr;
+	}();
 
 	const auto content = [&]() -> Ui::Premium::TopBarAbstract* {
 		if (peerWithPremium) {
-			return Ui::CreateChild<TopBarUser>(
+			return Ui::CreateChild<TopBarWithSticker>(
 				parent.get(),
 				_controller,
 				peerWithPremium,
 				_showFinished.events());
+		} else if (premiumGift) {
+			return Ui::CreateChild<TopBarWithSticker>(
+				parent.get(),
+				_controller,
+				TopBarWithStickerArgs{
+					.stickerValue = rpl::single(premiumGift),
+					.nameValue = tr::lng_gift_premium_title(),
+					.aboutValue = tr::lng_gift_premium_text(
+						lt_count,
+						rpl::single(premiumGiftData.perUserTotal * 1.),
+						Ui::Text::RichLangValue),
+					.type = TopBarWithStickerType::PremiumGift,
+				},
+				_showFinished.events());
 		}
 		const auto weak = base::make_weak(_controller);
 		const auto clickContextOther = [=] {
@@ -1102,7 +1214,7 @@ QPointer<Ui::RpWidget> Premium::createPinnedToTop(
 	}, content->lifetime());
 
 	const auto calculateMaximumHeight = [=] {
-		return isEmojiStatus
+		return (isEmojiStatus || isPremiumGift)
 			? st::settingsPremiumUserHeight + TopTransitionSkip()
 			: st::settingsPremiumTopHeight;
 	};
@@ -1168,14 +1280,14 @@ QPointer<Ui::RpWidget> Premium::createPinnedToTop(
 		}
 	}, content->lifetime());
 
-	return Ui::MakeWeak(not_null<Ui::RpWidget*>{ content });
+	return base::make_weak(not_null<Ui::RpWidget*>{ content });
 }
 
 void Premium::showFinished() {
 	_showFinished.fire({});
 }
 
-QPointer<Ui::RpWidget> Premium::createPinnedToBottom(
+base::weak_qptr<Ui::RpWidget> Premium::createPinnedToBottom(
 		not_null<Ui::RpWidget*> parent) {
 	const auto content = Ui::CreateChild<Ui::RpWidget>(parent.get());
 
@@ -1278,7 +1390,7 @@ QPointer<Ui::RpWidget> Premium::createPinnedToBottom(
 		_subscribe->setVisible(!premium && premiumPossible);
 	}, _subscribe->lifetime());
 
-	return Ui::MakeWeak(not_null<Ui::RpWidget*>{ content });
+	return base::make_weak(not_null<Ui::RpWidget*>{ content });
 }
 
 } // namespace
@@ -1352,6 +1464,15 @@ void ShowEmojiStatusPremium(
 	}
 }
 
+void ShowPremiumGiftPremium(
+		not_null<Window::SessionController*> controller,
+		const Data::StarGift &gift) {
+	ShowPremium(controller, Ref::PremiumGift::Serialize({
+		.documentId = gift.document->id,
+		.perUserTotal = gift.perUserTotal,
+	}));
+}
+
 void StartPremiumPayment(
 		not_null<Window::SessionController*> controller,
 		const QString &ref) {
