@@ -204,7 +204,8 @@ AdminLog::OwnedItem GenerateForwardedItem(
 		MTPlong(), // effect
 		MTPFactCheck(),
 		MTPint(), // report_delivery_until_date
-		MTPlong() // paid_message_stars
+		MTPlong(), // paid_message_stars
+		MTPSuggestedPost()
 	).match([&](const MTPDmessage &data) {
 		return history->makeMessage(
 			history->nextNonHistoryEntryId(),
@@ -616,8 +617,8 @@ object_ptr<Ui::RpWidget> PhoneNumberPrivacyController::setupMiddleWidget(
 }
 
 void PhoneNumberPrivacyController::saveAdditional() {
-	if (_saveAdditional) {
-		_saveAdditional();
+	if (const auto onstack = _saveAdditional) {
+		onstack();
 	}
 }
 
@@ -1296,8 +1297,8 @@ object_ptr<Ui::RpWidget> ProfilePhotoPrivacyController::setupMiddleWidget(
 }
 
 void ProfilePhotoPrivacyController::saveAdditional() {
-	if (_saveAdditional) {
-		_saveAdditional();
+	if (const auto onstack = _saveAdditional) {
+		onstack();
 	}
 }
 
@@ -1581,6 +1582,13 @@ object_ptr<Ui::RpWidget> BirthdayPrivacyController::setupAboveWidget(
 	return result;
 }
 
+struct GiftsAutoSavePrivacyController::AdditionalState {
+	Api::DisallowedGiftTypes disallowed;
+	rpl::event_stream<> disables;
+	Fn<void()> promo;
+	Fn<void()> save;
+};
+
 UserPrivacy::Key GiftsAutoSavePrivacyController::key() const {
 	return Key::GiftsAutoSave;
 }
@@ -1626,4 +1634,145 @@ bool GiftsAutoSavePrivacyController::allowMiniAppsToggle(
 	return true;
 }
 
+void GiftsAutoSavePrivacyController::ensureAdditionalState(
+		not_null<Window::SessionController*> controller,
+		rpl::lifetime &on) {
+	if (_state) {
+		return;
+	}
+	const auto session = &controller->session();
+	const auto globalPrivacy = &session->api().globalPrivacy();
+
+	_state = on.make_state<AdditionalState>();
+	_state->disallowed = globalPrivacy->disallowedGiftTypesCurrent();
+	_state->promo = [=] {
+		_state->disables.fire({});
+		const auto link = Ui::Text::Bold(
+			tr::lng_settings_generic_subscribe_link(tr::now));
+		Settings::ShowPremiumPromoToast(
+			controller->uiShow(),
+			tr::lng_settings_generic_subscribe(
+				tr::now,
+				lt_link,
+				Ui::Text::Link(link),
+				Ui::Text::WithEntities),
+			u"gifts_privacy"_q);
+	};
+	_state->save = [=] {
+		const auto now = _state->disallowed;
+		if (!session->premium()) {
+			return;
+		} else if (globalPrivacy->disallowedGiftTypesCurrent() == now) {
+			return;
+		} else {
+			globalPrivacy->updateDisallowedGiftTypes(now);
+		}
+	};
+}
+
+object_ptr<Ui::RpWidget> GiftsAutoSavePrivacyController::setupAboveWidget(
+		not_null<Window::SessionController*> controller,
+		not_null<QWidget*> parent,
+		rpl::producer<Option> optionValue,
+		not_null<QWidget*> outerContainer) {
+	auto result = object_ptr<Ui::VerticalLayout>(parent);
+	const auto content = result.data();
+
+	ensureAdditionalState(controller, content->lifetime());
+	using Type = Api::DisallowedGiftType;
+
+	const auto session = &controller->session();
+	const auto icon = content->add(object_ptr<Ui::SettingsButton>(
+		content,
+		tr::lng_edit_privacy_gifts_show_icon(),
+		st::settingsButtonNoIconLocked));
+	icon->toggleOn(rpl::single(
+		session->premium() && (_state->disallowed & Type::SendHide)
+	) | rpl::then(_state->disables.events() | rpl::map([=] {
+		return false;
+	})));
+	Data::AmPremiumValue(session) | rpl::start_with_next([=](bool value) {
+		icon->setToggleLocked(!value);
+		if (!value) {
+			_state->disables.fire({});
+		}
+	}, icon->lifetime());
+	icon->toggledValue() | rpl::start_with_next([=](bool enable) {
+		if (!enable) {
+			_state->disallowed &= ~Type::SendHide;
+		} else if (!session->premium()) {
+			_state->promo();
+		} else {
+			_state->disallowed |= Type::SendHide;
+		}
+	}, icon->lifetime());
+	Ui::AddSkip(content);
+	Ui::AddDividerText(
+		content,
+		tr::lng_edit_privacy_gifts_show_icon_about(
+			lt_emoji,
+			rpl::single(Ui::Text::IconEmoji(&st::settingsGiftIconEmoji)),
+			Ui::Text::WithEntities));
+
+	return result;
+}
+
+object_ptr<Ui::RpWidget> GiftsAutoSavePrivacyController::setupBelowWidget(
+		not_null<Window::SessionController*> controller,
+		not_null<QWidget*> parent,
+		rpl::producer<Option> option) {
+	auto result = object_ptr<Ui::VerticalLayout>(parent);
+	const auto content = result.data();
+
+	ensureAdditionalState(controller, content->lifetime());
+	using Type = Api::DisallowedGiftType;
+
+	const auto session = &controller->session();
+	auto premium = Data::AmPremiumValue(session);
+
+	Ui::AddSkip(content, st::settingsPeerToPeerSkip);
+	Ui::AddSubsectionTitle(
+		content,
+		tr::lng_edit_privacy_gifts_types());
+	const auto types = base::flat_map<Type, rpl::producer<QString>>{
+		{ Type::Limited, tr::lng_edit_privacy_gifts_limited() },
+		{ Type::Unlimited, tr::lng_edit_privacy_gifts_unlimited() },
+		{ Type::Unique, tr::lng_edit_privacy_gifts_unique() },
+		{ Type::Premium, tr::lng_edit_privacy_gifts_premium() },
+	};
+	for (const auto &[type, title] : types) {
+		const auto button = content->add(object_ptr<Ui::SettingsButton>(
+			content,
+			rpl::duplicate(title),
+			st::settingsButtonNoIconLocked));
+		button->toggleOn(rpl::single(
+			!session->premium() || !(_state->disallowed & type)
+		) | rpl::then(_state->disables.events() | rpl::map([=] {
+			return true;
+		})));
+		rpl::duplicate(premium) | rpl::start_with_next([=](bool value) {
+			button->setToggleLocked(!value);
+		}, button->lifetime());
+		button->toggledValue() | rpl::start_with_next([=](bool enable) {
+			if (enable) {
+				_state->disallowed &= ~type;
+			} else if (!session->premium()) {
+				_state->promo();
+			} else {
+				_state->disallowed |= type;
+			}
+		}, button->lifetime());
+	}
+	Ui::AddSkip(content);
+	Ui::AddDividerText(content, tr::lng_edit_privacy_gifts_types_about());
+
+	return result;
+}
+
+void GiftsAutoSavePrivacyController::saveAdditional() {
+	if (const auto onstack = _state->save) {
+		onstack();
+	}
+}
+
 } // namespace Settings
