@@ -31,6 +31,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/self_destruction_box.h"
 #include "core/application.h"
 #include "core/core_settings.h"
+#include "history/view/media/history_view_media_common.h"
 #include "ui/chat/chat_style.h"
 #include "ui/effects/premium_graphics.h"
 #include "ui/effects/premium_top_bar.h"
@@ -605,6 +606,19 @@ void SetupLoginEmail(
 
 	button->addClickHandler([=, email = std::move(email)] {
 		controller->uiShow()->show(Box([=](not_null<Ui::GenericBox*> box) {
+			{
+				box->getDelegate()->setTitle(rpl::duplicate(
+					email
+				) | rpl::map(Ui::Text::WrapEmailPattern));
+				for (const auto &child : ranges::views::reverse(
+						box->parentWidget()->children())) {
+					if (child && child->isWidgetType()) {
+						(static_cast<QWidget*>(child))->setAttribute(
+							Qt::WA_TransparentForMouseEvents);
+						break;
+					}
+				}
+			}
 			Ui::ConfirmBox(box, Ui::ConfirmBoxArgs{
 				.text = tr::lng_settings_cloud_login_email_box_about(),
 				.confirmed = [=](Fn<void()> close) {
@@ -613,9 +627,6 @@ void SetupLoginEmail(
 				},
 				.confirmText = tr::lng_settings_cloud_login_email_box_ok(),
 			});
-			box->getDelegate()->setTitle(rpl::duplicate(
-				email
-			) | rpl::map(Ui::Text::WrapEmailPattern));
 		}));
 	});
 
@@ -971,7 +982,9 @@ void SetupSensitiveContent(
 	Ui::AddSkip(inner);
 	Ui::AddSubsectionTitle(inner, tr::lng_settings_sensitive_title());
 
+	const auto show = controller->uiShow();
 	const auto session = &controller->session();
+	const auto disable = inner->lifetime().make_state<rpl::event_stream<>>();
 
 	std::move(
 		updateTrigger
@@ -982,13 +995,23 @@ void SetupSensitiveContent(
 		inner,
 		tr::lng_settings_sensitive_disable_filtering(),
 		st::settingsButtonNoIcon
-	))->toggleOn(
-		session->api().sensitiveContent().enabled()
-	)->toggledChanges(
+	))->toggleOn(rpl::merge(
+		session->api().sensitiveContent().enabled(),
+		disable->events() | rpl::map_to(false)
+	))->toggledChanges(
 	) | rpl::filter([=](bool toggled) {
 		return toggled != session->api().sensitiveContent().enabledCurrent();
 	}) | rpl::start_with_next([=](bool toggled) {
-		session->api().sensitiveContent().update(toggled);
+		if (toggled && session->appConfig().ageVerifyNeeded()) {
+			disable->fire({});
+
+			HistoryView::ShowAgeVerificationRequired(
+				show,
+				session,
+				[] {});
+		} else {
+			session->api().sensitiveContent().update(toggled);
+		}
 	}, container->lifetime());
 
 	Ui::AddSkip(inner);
