@@ -23,6 +23,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/painter.h"
 #include "ui/power_saving.h"
 #include "ui/ui_utility.h"
+#include "data/data_saved_sublist.h"
 #include "data/data_session.h"
 #include "data/data_forum_topic.h"
 #include "data/stickers/data_custom_emoji.h"
@@ -236,6 +237,7 @@ void Manager::showNextFromQueue() {
 			this,
 			queued.history,
 			queued.topicRootId,
+			queued.monoforumPeerId,
 			queued.peer,
 			queued.author,
 			queued.item,
@@ -383,7 +385,25 @@ void Manager::doClearFromTopic(not_null<Data::ForumTopic*> topic) {
 		}
 	}
 	for (const auto &notification : _notifications) {
-		if (notification->unlinkHistory(history, topicRootId)) {
+		if (notification->unlinkHistory(history, topicRootId, PeerId())) {
+			_positionsOutdated = true;
+		}
+	}
+	showNextFromQueue();
+}
+
+void Manager::doClearFromSublist(not_null<Data::SavedSublist*> sublist) {
+	const auto history = sublist->owningHistory();
+	const auto sublistPeerId = sublist->sublistPeer()->id;
+	for (auto i = _queuedNotifications.begin(); i != _queuedNotifications.cend();) {
+		if (i->history == history && i->monoforumPeerId == sublistPeerId) {
+			i = _queuedNotifications.erase(i);
+		} else {
+			++i;
+		}
+	}
+	for (const auto &notification : _notifications) {
+		if (notification->unlinkHistory(history, MsgId(), sublistPeerId)) {
 			_positionsOutdated = true;
 		}
 	}
@@ -618,6 +638,7 @@ Notification::Notification(
 	not_null<Manager*> manager,
 	not_null<History*> history,
 	MsgId topicRootId,
+	PeerId monoforumPeerId,
 	not_null<PeerData*> peer,
 	const QString &author,
 	HistoryItem *item,
@@ -633,7 +654,9 @@ Notification::Notification(
 , _history(history)
 , _topic(history->peer->forumTopicFor(topicRootId))
 , _topicRootId(topicRootId)
-, _userpicView(_peer->createUserpicView())
+, _sublist(history->peer->monoforumSublistFor(monoforumPeerId))
+, _monoforumPeerId(monoforumPeerId)
+, _userpicView(_peer->userpicPaintingPeer()->createUserpicView())
 , _author(author)
 , _reaction(reaction)
 , _item(item)
@@ -1149,10 +1172,14 @@ void Notification::changeHeight(int newHeight) {
 	manager()->changeNotificationHeight(this, newHeight);
 }
 
-bool Notification::unlinkHistory(History *history, MsgId topicRootId) {
+bool Notification::unlinkHistory(
+		History *history,
+		MsgId topicRootId,
+		PeerId monoforumPeerId) {
 	const auto unlink = _history
 		&& (history == _history || !history)
-		&& (topicRootId == _topicRootId || !topicRootId);
+		&& (topicRootId == _topicRootId || !topicRootId)
+		&& (monoforumPeerId == _monoforumPeerId || !monoforumPeerId);
 	if (unlink) {
 		hideFast();
 		_history = nullptr;
@@ -1202,7 +1229,9 @@ void Notification::mousePressEvent(QMouseEvent *e) {
 		unlinkHistoryInManager();
 	} else {
 		e->ignore();
-		manager()->notificationActivated(myId());
+		manager()->notificationActivated(myId(), {
+			.allowNewWindow = true,
+		});
 	}
 }
 
