@@ -30,6 +30,10 @@ class SavedMessages;
 enum class StorySourcesList : uchar;
 } // namespace Data
 
+namespace Dialogs {
+struct SearchState;
+} // namespace Dialogs
+
 namespace ChatHelpers {
 class TabbedSelector;
 class EmojiInteractions;
@@ -162,8 +166,9 @@ struct SectionShow {
 		return copy;
 	}
 
-	TextWithEntities highlightPart;
+	MessageHighlightId highlight;
 	int highlightPartOffsetHint = 0;
+	int highlightTodoItemId = 0;
 	std::optional<TimeId> videoTimestamp;
 	Way way = Way::Forward;
 	anim::type animated = anim::type::normal;
@@ -178,6 +183,8 @@ struct SectionShow {
 
 };
 
+[[nodiscard]] MessageHighlightId SearchHighlightId(const QString &query);
+
 class SessionController;
 
 class SessionNavigation : public base::has_weak_ptr {
@@ -383,7 +390,7 @@ public:
 	void stickerOrEmojiChosen(FileChosen chosen);
 	[[nodiscard]] rpl::producer<FileChosen> stickerOrEmojiChosen() const;
 
-	QPointer<Ui::BoxContent> show(
+	base::weak_qptr<Ui::BoxContent> show(
 		object_ptr<Ui::BoxContent> content,
 		Ui::LayerOptions options = Ui::LayerOption::KeepOther,
 		anim::type animated = anim::type::normal);
@@ -404,7 +411,7 @@ public:
 	void setSearchInChat(Dialogs::Key value) {
 		_searchInChat = value;
 	}
-	bool uniqueChatsInSearchResults() const;
+	bool uniqueChatsInSearchResults(const Dialogs::SearchState &state) const;
 
 	void openFolder(not_null<Data::Folder*> folder);
 	void closeFolder();
