@@ -572,10 +572,7 @@ int CustomSimulcastEncoderAdapter::Encode(
     // correctly sample/scale the source texture.
     // TODO(perkj): ensure that works going forward, and figure out how this
     // affects webrtc:5683.
-    if ((layer.width() == src_width && layer.height() == src_height) ||
-        (input_image.video_frame_buffer()->type() ==
-             VideoFrameBuffer::Type::kNative &&
-         layer.encoder().GetEncoderInfo().supports_native_handle)) {
+    if (layer.width() == src_width && layer.height() == src_height) {
       int ret = layer.encoder().Encode(input_image, &stream_frame_types);
       if (ret != WEBRTC_VIDEO_CODEC_OK) {
         return ret;
