@@ -113,7 +113,10 @@ private:
 
 class StaticCustomEmoji final : public CustomEmoji {
 public:
-	StaticCustomEmoji(QImage &&image, QString entity);
+	StaticCustomEmoji(
+		QImage &&image,
+		QString entity,
+		QMargins padding = {});
 
 	int width() override;
 	QString entityData() override;
@@ -125,6 +128,32 @@ public:
 private:
 	QImage _image;
 	QString _entity;
+	QMargins _padding;
+
+};
+
+class PaletteDependentCustomEmoji final : public CustomEmoji {
+public:
+	PaletteDependentCustomEmoji(
+		Fn<QImage()> factory,
+		QString entity,
+		QMargins padding = {});
+
+	int width() override;
+	QString entityData() override;
+	void paint(QPainter &p, const Context &context) override;
+	void unload() override;
+	bool ready() override;
+	bool readyInDefaultState() override;
+
+private:
+	void validateFrame();
+
+	Fn<QImage()> _factory;
+	QString _entity;
+	QMargins _padding;
+	QImage _frame;
+	int _paletteVersion = 0;
 
 };
 
