@@ -14,7 +14,6 @@
 #include "ui/basic_click_handlers.h" // UrlClickHandler
 #include "ui/inactive_press.h"
 #include "ui/painter.h"
-#include "ui/qt_weak_factory.h"
 #include "ui/integration.h"
 #include "ui/ui_utility.h"
 #include "base/qt/qt_common_adapters.h"
@@ -268,7 +267,7 @@ void FlatLabel::textUpdated() {
 	refreshSize();
 	setMouseTracking(_selectable || _text.hasLinks());
 	if (_text.hasSpoilers()) {
-		_text.setSpoilerLinkFilter([weak = Ui::MakeWeak(this)](
+		_text.setSpoilerLinkFilter([weak = base::make_weak(this)](
 				const ClickContext &context) {
 			return (context.button == Qt::LeftButton) && weak;
 		});
@@ -658,7 +657,7 @@ void FlatLabel::touchEvent(QTouchEvent *e) {
 	case QEvent::TouchEnd: {
 		if (!_touchInProgress) return;
 		_touchInProgress = false;
-		auto weak = MakeWeak(this);
+		auto weak = base::make_weak(this);
 		if (_touchSelect) {
 			dragActionFinish(_touchPos, Qt::RightButton);
 			QContextMenuEvent contextMenu(QContextMenuEvent::Mouse, mapFromGlobal(_touchPos), _touchPos);
