@@ -7,7 +7,6 @@
 #include "ui/widgets/scroll_area.h"
 
 #include "ui/painter.h"
-#include "ui/qt_weak_factory.h"
 #include "ui/ui_utility.h"
 #include "base/qt/qt_common_adapters.h"
 #include "base/debug_log.h"
@@ -663,7 +662,7 @@ void ScrollArea::touchEvent(QTouchEvent *e) {
 	case QEvent::TouchEnd: {
 		if (!_touchPress) return;
 		_touchPress = false;
-		auto weak = MakeWeak(this);
+		auto weak = base::make_weak(this);
 		if (_touchScroll) {
 			if (_touchScrollState == TouchScrollState::Manual) {
 				_touchScrollState = TouchScrollState::Auto;
